/******************************************************************************
*******************************************************************************
**                                                                           **
**      C Swing - A graphical file and directory maintenance utility         **
**                                                                           **
*******************************************************************************
**                                                                           **
**      Original C coding by: S. J. Brown                                    **
**                                                                           **
**      Modifications by many others; see the documentation for a list       **
**                                                                           **
**      Copyright 1994, Harry Flowers and Foteos Macrides                    **
**                                                                           **
**      Terms of Copyright:  This software may be freely distributed         **
**          and used by anyone, providing the distributor provides it        **
**          free of charge or at a price which only covers distribution      **
**          costs.  It may not be sold.  Note that versions prior to         **
**          V3.7.4 were public domain, and are not subject to these terms.   **
**                                                                           **
**      Disclaimer: No warranties as to the use or reliability of this       **
**          software are given by the authors or distributors.  Use at       **
**          your own risk.                                                   **
**                                                                           **
*******************************************************************************
******************************************************************************/
 
#include <quidef.h>
#include <jbcmsgdef.h>
#ifndef $DESCRIPTOR
#include <descrip.h>
#endif
#include <signal.h>

#define MAX_PRINT_QUEUES    50
#define PRINT_MENU_WIDTH    78
#define QUE_NAME_LEN	    15
#define QUE_DESC_LEN	    50
#define QUE_DESC_DELIM	    '`'  /*  Delimiter which marks end of formats */
#define QUE_STAT_LEN	    8

/*	  
**  The following descriptor definition is similar to the Array
**  descriptor in descrip.h, but it includes the optional blocks.
**  It was obtained from Digital Customer Support in an example
**  on how to call SMG$CREATE_MENU
*/	  

struct dsc$descriptor_a1 {
	unsigned short	dsc$w_length;	/* length of an array element in bytes,
					     or if dsc$b_dtype is DSC$K_DTYPE_V, bits,
					     or if dsc$b_dtype is DSC$K_DTYPE_P, digits (4 bits each) */
	unsigned char	dsc$b_dtype;	/* data type code */
	unsigned char	dsc$b_class;	/* descriptor class code = DSC$K_CLASS_A */
	char		*dsc$a_pointer;	/* address of first actual byte of data storage */
	char		dsc$b_scale;	/* signed power-of-two or -ten multiplier, as specified by
					     dsc$v_fl_binscale, to convert from internal to external form */
	unsigned char	dsc$b_digits;	/* if nonzero, number of decimal digits in internal representation */
	struct
	{
		unsigned		 : 3;	/* reserved;  must be zero */
		unsigned dsc$v_fl_binscale : 1; /* if set, dsc$b_scale is a power-of-two, otherwise, -ten */
		unsigned dsc$v_fl_redim	 : 1;	/* if set, indicates the array can be redimensioned */
		unsigned dsc$v_fl_column : 1;	/* if set, indicates column-major order (FORTRAN) */
		unsigned dsc$v_fl_coeff  : 1;	/* if set, indicates the multipliers block is present */
		unsigned dsc$v_fl_bounds : 1;	/* if set, indicates the bounds block is present */
	}		dsc$b_aflags;	/* array flag bits */
	unsigned char	dsc$b_dimct;	/* number of dimensions */
	unsigned long	dsc$l_arsize;	/* total size of array in bytes,
					     or if dsc$b_dtype is DSC$K_DTYPE_P, digits (4 bits each) */
	char		*dsc$a_a0;	/* add of ele w/ zero coefficients */
	long		dsc$l_m;	/*  Addressing coefficients (multipliers) */
	struct {
	    long dsc$l_l;		/* lower bound */
	    long dsc$l_u;		/* upper bound */
	} dsc$bounds[1];
};

/* macro to build descriptor */
#define DESC_ARRAY_1DIM(name,ptr,num_opts,elmt_size) \
    struct dsc$descriptor_a1 name = \
	   {elmt_size, DSC$K_DTYPE_T, DSC$K_CLASS_A, ptr, 0, 0, \
	   {0,0,0,0,1,1}, 1, num_opts * elmt_size, ptr, 1, 0, num_opts - 1}

#define UNKNOWN 0

/*	  
**  This section defines the extensions to look for and the queue
**  designations to associate them with.
*/	  
struct queextpair {
    unsigned char   extension[32];
    unsigned char   designation[6];
};

#define NUM_EXTS    18

struct queextpair   extmappings[NUM_EXTS] = {
    { "PS",	    "PS" },
    { "EPS",	    "PS" },
    { "PCL",	    "PCL" },
    { "TXT",	    "TXT" },
    { "LIS",	    "TXT" },
    { "LOG",	    "TXT" },
    { "COM",	    "TXT" },
    { "MEM",	    "TXT" },
    { "DOC",	    "TXT" },
    { "README",	    "TXT" },
    { "1ST",	    "TXT" },
    { "C",	    "TXT" },
    { "H",	    "TXT" },
    { "FOR",	    "TXT" },
    { "INC",	    "TXT" },
    { "PAS",	    "TXT" },
    { "SAS",	    "TXT" },
    { "ZET",	    "ZET" }
};

struct printqueue {
    char name[QUE_NAME_LEN+1];
    char desc[QUE_DESC_LEN+1];
    char status[QUE_STAT_LEN+1];
    int jobs;
};
