/******************************************************************************
*******************************************************************************
**                                                                           **
**      C Swing - A graphical file and directory maintenance utility         **
**                                                                           **
*******************************************************************************
**                                                                           **
**      Original C coding by: S. J. Brown                                    **
**                                                                           **
**      Modifications by many others; see the documentation for a list       **
**                                                                           **
**      Copyright 1994, Harry Flowers and Foteos Macrides                    **
**                                                                           **
**      Terms of Copyright:  This software may be freely distributed         **
**          and used by anyone, providing the distributor provides it        **
**          free of charge or at a price which only covers distribution      **
**          costs.  It may not be sold.  Note that versions prior to         **
**          V3.7.4 were public domain, and are not subject to these terms.   **
**                                                                           **
**      Disclaimer: No warranties as to the use or reliability of this       **
**          software are given by the authors or distributors.  Use at       **
**          your own risk.                                                   **
**                                                                           **
*******************************************************************************
******************************************************************************/
 
/*
**  swing_17.c
*/
#include <stdio.h>
#include <iodef.h>
#include <rms.h>
#include <ssdef.h>
#include <smgdef.h>
#include <smgmsg.h>
#include <dvidef.h>
#include <fchdef.h>
#include <atrdef.h>
#include <fibdef.h>
#include "more_descrip.h"
#include "swing.h"
#include "swing_refs.h"

#define EXAMINED_WEDGE			50
#define DELETE_SCREEN_EXTRA_ROWS	4
#define DELETE_SCREEN_EXTRA_COLS	8
#define DELETE_SCREEN_EXTRA_X		4
#define DELETE_SCREEN_EXTRA_Y		2

static char delete_device[SPEC_SIZE+1];
static int return_length, errcnt, freebk, maxbk;
static char volnam[100];
static int delete_screen, display_delete_blank;

static struct {
short buffer_length;
short item_code;
int buffer_address;
int return_length_address;
} itmlst[5] =  {4,	DVI$_ERRCNT, 		&errcnt,	&return_length,
		4,	DVI$_FREEBLOCKS,	&freebk,	&return_length,
		4,	DVI$_MAXBLOCK,		&maxbk,		&return_length,
                12,     DVI$_VOLNAM,		&volnam[0],     &return_length,
                0,	0,			0,		0,
};


/*
**  acb elements
*/
unsigned long uchar;

/*
**  acb descriptor for io$_access
*/
ACB acb1[] =   {
               /*
               **  file characteristics (directory ??)
               */
               ATR$S_UCHAR,     ATR$C_UCHAR,     &uchar,

               /*
               **  terminate the list
               */
               0,               0,               0};


/*
**  a blank fib used for initialising other fibs
*/
struct fibdef blank_fib;

/*
**  initialise the SMG side of the delete operation
*/
delete_screen_init()
{
    /*
    **  initialize the delete screen display if it's not already been
    **  initialized
    */
    int status, rows, cols;

    if (delete_screen_initialized)
        return(TRUE);

    delete_screen_initialized = TRUE;

    status = smg$create_virtual_display(&DELETE_SCREEN_ROWS,
                                        &DELETE_SCREEN_COLUMNS,
                                        &display_delete,
                                        &SMG$M_BORDER);
    if (status != SS$_NORMAL)
        panic(status,"create virtual display","delete_screen_init");

    rows = DELETE_SCREEN_ROWS + DELETE_SCREEN_EXTRA_ROWS;
    cols = DELETE_SCREEN_COLUMNS + DELETE_SCREEN_EXTRA_COLS;

    status = smg$create_virtual_display(&rows,
                                        &cols,
                                        &display_delete_blank);
    if (status != SS$_NORMAL)
        panic(status,"create virtual blank display","delete_screen_init");
}

/*
**  slam out the delete screen
*/
delete_screen_on(text)
char *text;
{
    int status,x,y;

    /*
    **  display (init if nec) the delete screen
    */
    delete_screen_init();

    /*
    **  get current device - we'll use this later
    */
    translate("SYS$DISK",delete_device);

    smg$set_cursor_mode(&paste,&1);

    label_delete_screen(text);

    status = smg$erase_display(&display_delete);
    if (status != SS$_NORMAL)
        panic(status,"erase display","delete_screen_on");

    /*
    **  reset vars
    */
    deleted_ok = 0;
    not_deleted_ok = 0;
    total_examined = 0;
    originally_ok = -1;

    smg$begin_pasteboard_update(&paste);

    delete_screen_status("* STARTING UP *",
                         deleted_ok,
                         not_deleted_ok,
                         total_examined,
                         TRUE,
			 text);        

    x = DELETE_SCREEN_X - DELETE_SCREEN_EXTRA_X;
    y = DELETE_SCREEN_Y - DELETE_SCREEN_EXTRA_Y;

    smg$paste_virtual_display(&display_delete_blank, 
                              &paste,
                              &y, 
                              &x);

    smg$paste_virtual_display(&display_delete, 
                              &paste,
                              &DELETE_SCREEN_Y, 
                              &DELETE_SCREEN_X);

    smg$end_pasteboard_update(&paste);
}

/*
**  label the screen
*/
label_delete_screen(text)
char *text;
{
    int status;
    unsigned long used, perm, over;
    char label[100];

    strcpy(label,text);

    status = get_user_quota("SYS$DISK",
                            &used,
                            &perm,
                            &over);
    if (status == SS$_NORMAL)
        sprintf(&label[strlen(label)],"    QUOTA %d USED %d ",perm,used);

    status = smg$label_border(&display_delete,
                              DSC1(label),
                              &SMG$K_TOP,
                              0,
                              &(SMG$M_REVERSE));
    if (status != SS$_NORMAL)
        panic(status,"label border display_delete","label_delete_screen");
}

/*
**  away with the delete screen and restore the display
*/
delete_screen_off()
{
    /*
    **  remove the delete screen
    */
    int status;

    smg$begin_pasteboard_update(&paste);

    smg$unpaste_virtual_display(&display_delete, 
                                &paste);

    smg$unpaste_virtual_display(&display_delete_blank, 
                                &paste);

    smg$end_pasteboard_update(&paste);

    smg$set_cursor_mode(&paste,&0);
}

/*
**  update the status of files deleted, not deleted etc.
*/
delete_screen_status(filename,ok,not_ok,tot,opt,text)
char *filename;
int ok, not_ok, tot, opt;
{
    /*
    **  update the delete screen display
    */
    int i,status,row,col;
    float size,left,used,percent;
    char file[SPEC_SIZE+1];

    label_delete_screen(text);
    
    /*
    **  only update filename if opt == TRUE
    */
    if (opt)
    {
        /*
        **  strip out the dir part of file name - preset file to filename
        **  in case no dir part supplied
        */
        strcpy(file,filename);
    
        for (i = strlen(filename); i > 0; i--)
            if (filename[i] == ']')
            {
                strcpy(file,&filename[i+1]);
                break;
            }
    
        smg$set_cursor_abs(&display_delete,&DELETE_FILE_Y,&DELETE_FILE_X);
        smg$put_chars(&display_delete,DSC1("FILE "));
        sprintf(junk," %s ",file);
        smg$put_chars(&display_delete,DSC1(junk),0,0,0,0);
        smg$erase_line(&display_delete);
    
        /*
        **  now let's make a summary of our device if we've deleted anything
        **  or this is the first call
        */
        if (ok != originally_ok)
        {
            originally_ok = ok;
    
            status = sys$getdviw(0,				/* efn */
            	 	     	 0,				/* chan */
        	    		 DSC1(delete_device),		/* devnam */
        	   		 &itmlst[0].buffer_length,	/* itmlst */
        			 0,
        			 0,
        			 0,
        			 0);
            if (status == SS$_NORMAL)
            {
           	size = (float)maxbk;
                size = size / 2048;
        
                left = (float)freebk;
                left = left / 2048;
        
                used = size - left;
                if (size > 0.0)
                    percent = (left * 100)/size;
                else
                    percent = 0.0;
    
                smg$set_cursor_abs(&display_delete,&DELETE_STAT_Y,&DELETE_STAT_X);
                sprintf(junk,"Current Device: %s %s",delete_device,volnam);
                smg$put_line(&display_delete,DSC1(junk));

                /*
		**  show current usage
                */
                smg$set_cursor_abs(&display_delete,&DELETE_STAT1_Y,&DELETE_STAT1_X);
                sprintf(junk,
                        "Size: %4.2fmb  Used: %4.2fmb  Free: %4.2fmb (%3.2f%c)",
                        size,
                        used,
                        left,
                        percent,
                        '%c');
                smg$put_line(&display_delete,DSC1(junk));
            }    				
        }
    }

    smg$set_cursor_abs(&display_delete,&DELETE_STATUS_Y,&DELETE_STATUS_X);
    sprintf(junk,"Total files examined %d",tot);
    smg$put_line(&display_delete,DSC1(junk));
    
    if (ok || not_ok)
    {
        smg$set_cursor_abs(&display_delete,&DELETE_STATUS1_Y,&DELETE_STATUS1_X);
        if (deleted_option == PURGE)
            sprintf(junk,"Purged successfully %d",ok);
        else
            sprintf(junk,"Deleted successfully %d",ok);
        smg$put_line(&display_delete,DSC1(junk));
    }

    if (not_ok)
    {
        smg$set_cursor_abs(&display_delete,&DELETE_STATUS2_Y,&DELETE_STATUS2_X);
        if (deleted_option == PURGE)
            sprintf(junk,"Failed to purge %d files",not_ok);
        else
            sprintf(junk,"Failed to delete %d files",not_ok);
        smg$put_line(&display_delete,DSC1(junk));
    }
    smg$set_cursor_abs(&display_delete,&DELETE_SCREEN_ROWS,&DELETE_SCREEN_COLUMNS);
}


/*
**  output a message to the status line of the delete screen
*/
delete_screen_message(text)
char *text;
{
    smg$set_cursor_abs(&display_delete,&DELETE_MSG_Y,&DELETE_MSG_X);
    smg$put_line(&display_delete,DSC1(text),0,0);
}

/*
**  delete files in dir n (NODE structure)
**
**  if sub_dirs then go down the dirs
**  option == DELETE then we delete all files, otherwise
**  option == PURGE, and we just purge the directories
*/
delete_many_files(n,sub_dirs,option,spec)
NODE *n;
int sub_dirs;
int option;
char *spec;
{
    char text [256];
    int channel, status;

    /*
    **  make sure this is a kosher directory
    */
    if (!set_default(n->dir_name))
        return(FALSE);

    /*
    **  assign a channel to the current device
    */
    channel = 0;
    status = sys$assign(DSC1("SYS$DISK"),
                        &channel,
                        0,
                        0);
    if (status != SS$_NORMAL)
        panic(status,"sys$asign","delete_many_files");

    /*
    **  display delete screen with the desired title
    */
    if (option == PURGE)
    {
        if (sub_dirs)
            sprintf(text," PURGING [...]*.*;* ");
        else
            sprintf(text," PURGING %s ",spec);
    }
    else
    {
        if (sub_dirs)
            strcpy(text," DELETING [...]*.*;* ");
        else
            strcpy(text," DELETING ");
    }

    deleted_option = option;

    delete_screen_on(text);

    /*
    **  now do the deleting or purging, as selected
    */
    delete_many_more_files(n->fid,
                           channel,
                           sub_dirs,
                           option,
                           spec,
                           text);

    /*
    **  if we're deleteing, then we delete the starting directory as well
    */
    if (option == DELETE)
    {
        status = delete_a_file_by_fid(n->fid,
                                      n->did,
                                     "SYS$DISK");

        if (status)
            deleted_ok++;
        else
            not_deleted_ok++;

        total_examined++;
    }
    /*
    **  say it's over, get user to acknowledge, restore display
    */
    delete_screen_status("* FINISHED *",
                         deleted_ok,
                         not_deleted_ok,
                         total_examined,
                         TRUE,
                         text);        

    smg$ring_bell(&display_delete);
    delete_screen_message(PRESS_TO_CONTINUE);
    get_key(ALLOW_AUTO);
    delete_screen_off();

    /*
    **  deassign the i/o channel
    */
    sys$dassgn(channel);

    return(TRUE);
}

/*
**  delete or purge files in a directory
**  specified by fid and channel
*/
delete_many_more_files(fid,channel,sub_dirs,option,spec,text)
short *fid;
int channel;
int sub_dirs;
int option;
char *spec;
char *text;
{
    int del_opt;
    int ver1, ver2, len1, len2, file_length;
    int purge, delete;
    int status, i, length;
    long iosb[2], last_wcc;
    char filename[256];
    char last_file[256];
    char spec_str[256];
    struct dsc$descriptor_s fib_desc;
    struct fibdef fib;

    /*
    **  initialize
    */

    strcpy(last_file,";");

    fib = blank_fib;
    last_wcc = 0;

    fib_desc.dsc$w_length  = FIB$C_LENGTH;
    fib_desc.dsc$a_pointer = &fib;
    fib_desc.dsc$b_class   = DSC$K_CLASS_S;
    fib_desc.dsc$b_dtype   = DSC$K_DTYPE_T;

#ifdef __DECC
    fib.fib$w_did[0] = fid[0];
    fib.fib$w_did[1] = fid[1];
    fib.fib$w_did[2] = fid[2];
    fib.fib$w_nmctl = FIB$M_WILD;
#else
    fib.fib$r_did_overlay.fib$w_did[0] = fid[0];
    fib.fib$r_did_overlay.fib$w_did[1] = fid[1];
    fib.fib$r_did_overlay.fib$w_did[2] = fid[2];
    fib.fib$r_nmctl_overlay.fib$w_nmctl = FIB$M_WILD;
#endif

    /*
    **  pull out all the files in the directory, checking to see if the
    **  files themselves are directories
    */
    while(1)
    {
        filename[0] = 0;
        file_length = 0;
	strcpy(spec_str, spec);
        uchar = 0;
        iosb[0] = 0;
        status = sys$qiow(0,
                          channel,
                          IO$_ACCESS,
                          &iosb[0],
                          0,
                          0,
                          &fib_desc,
                          DSC1(spec_str),
                          &file_length,
                          DSCA1(filename),
                          acb1,
                          0);
        if (status != SS$_NORMAL)
            panic(status,
                  "io$_access directory lookup",
                  "delete many more files");

        if (iosb[0] == SS$_NOMOREFILES)
        {
            return(TRUE);
        }

        /*
        **  file in dir but not indexf.sys
        */
        if (iosb[0] == SS$_NOSUCHFILE)
        {
            fib.fib$l_wcc++;
            continue;
        }

        /*
        **  on error go get next file unless fib.l_wcc == 0; in which case
        **  we can't read the dir, so we bomb
        */
        if (iosb[0] != SS$_NORMAL)
        {
            if (fib.fib$l_wcc == 0)
            {
                /*
                **  we can't read this directory - go home
                */
                warning(full_error_text(iosb[0]));
                return(iosb[0]);
            }

            /*
            **  assume not a dir
            */
            continue;
        }

        /*
        **  carry on
        */
        filename[file_length] = 0;

        /*
        **  if it was a directory (& not [000000].dir) we'll go and process it
        **  by calling this same function
        */
        if (uchar & FCH$M_DIRECTORY &&
#ifdef __DECC
            (fib.fib$w_fid[0] != mfd_did[0] ||
             fib.fib$w_fid[1] != mfd_did[1]) )
#else
            (fib.fib$r_fid_overlay.fib$w_fid[0] != mfd_did[0] ||
             fib.fib$r_fid_overlay.fib$w_fid[1] != mfd_did[1]) )
#endif
        {
            /*
            **  we've a directory, so call this function first if we're
            **  handling sub-directories
            */
            if (sub_dirs)
#ifdef __DECC
                delete_many_more_files(fib.fib$w_fid,
#else
                delete_many_more_files(fib.fib$r_fid_overlay.fib$w_fid,
#endif
                                       channel,
                                       sub_dirs,
                                       option,
                                       spec_str,
                                       text);
        }
        /*
        **  now process this file
        */
        total_examined++;
        del_opt = FALSE;

        /*
        **  if we're purging and this one wasn't a directory, try to
        **  purge it
        */
        if (option == PURGE && !(uchar & FCH$M_DIRECTORY))
        {
            /*
            **  let's see if this file is a junior version
            **  of the previous - if so it's purgeable
            */
            len1 = strcspn(filename,";");
            len2 = strcspn(last_file,";");
            ver1 = atoi(&filename[len1+1]);
            ver2 = atoi(&last_file[len2+1]);
            if (len1 == len2 && 
                strncmp(last_file,filename,len1) == 0 &&
                ver1 < ver2)
            {
                /*
                **  same name.type
                */
                del_opt = TRUE;
#ifdef __DECC
                i = delete_a_file_by_fid(fib.fib$w_fid,
                                         fib.fib$w_did,
#else
                i = delete_a_file_by_fid(fib.fib$r_fid_overlay.fib$w_fid,
                                         fib.fib$r_did_overlay.fib$w_did,
#endif
                                         "SYS$DISK");
            }
        } 
        else if (option == DELETE)
        {
            /*
            **  delete it
            */
            del_opt = TRUE;
#ifdef __DECC
            i = delete_a_file_by_fid(fib.fib$w_fid,
                                     fib.fib$w_did,
#else
            i = delete_a_file_by_fid(fib.fib$r_fid_overlay.fib$w_fid,
                                     fib.fib$r_did_overlay.fib$w_did,
#endif
                                     "SYS$DISK");
        }

        if (del_opt)
        {
            if (i)
            {
                /*
                **  Drop context back one - this is a kludge. The problem
                **  is that we'll miss files if we delete now, cos the
                **  context will be out. It would be better to build up a list
                **  of the files to be deleted at the end of the directory
                **  walk-through (perhaps).
                */
                fib.fib$l_wcc = last_wcc;
                deleted_ok++;
            }
            else
                not_deleted_ok++;
        }
        
        /*
        **  update the delete screen info if we've tried to delete or
        **  the no of files examined is a multiple of EXAMINED_WEDGE
        */
        if (del_opt || total_examined % EXAMINED_WEDGE == 0)
        {
            delete_screen_status(filename,
                                 deleted_ok,
                                 not_deleted_ok,
                                 total_examined,
                                 del_opt,
                                 text);
        }
        strcpy(last_file, filename);
	last_wcc = fib.fib$l_wcc;
    }
}

/*
**  delete the file fid on device removing it from directory did
**  use RMS (couldn't get sys$qiow to go) change protection to make sure
**  we can delete it
*/
delete_a_file_by_fid(fid, did, device)
short *fid;
short *did;
char *device;
{
    int status;
    short prot, dummy;
    struct FAB fab;
    struct NAM nam;

    fab = cc$rms_fab;
    nam = cc$rms_nam;

    fab.fab$b_fac = 0;
    fab.fab$l_fop = FAB$M_NAM;
    fab.fab$l_nam = &nam;

    nam.nam$w_fid[0] = fid[0];
    nam.nam$w_fid[1] = fid[1];
    nam.nam$w_fid[2] = fid[2];

    nam.nam$w_did[0] = did[0];
    nam.nam$w_did[1] = did[1];
    nam.nam$w_did[2] = did[2];

    sprintf(nam.nam$t_dvi,"%c%s",strlen(device),device);

    status = sys$erase(&fab);
    if (status != RMS$_SUC)
    {
#ifdef CHANGE_PROT_TO_DELETE
        /*
        **  remove any file protection
        */
        set_protection(device,fid,0,&prot);

	/*
	**  try again
	*/
        if (sys$erase(&fab) != RMS$_SUC)
        {
            /*
            **  failed - restore the protection
            */
            set_protection(device,fid,prot,&dummy);
            return(FALSE);
	}
#else
    return(FALSE);
#endif /* CHANGE_PROT_TO_DELETE */
    }
    return(TRUE);
}

/*
**  delete a file by standard file spec (dev:[dir]name.type;version)
*/
delete_a_file(spec)
char *spec;
{
    int status;

    /*
    **  delete
    */
    status = lib$delete_file(DSC1(spec));
        return(status);
}
