/******************************************************************************
*******************************************************************************
**                                                                           **
**      C Swing - A graphical file and directory maintenance utility         **
**                                                                           **
*******************************************************************************
**                                                                           **
**      Original C coding by: S. J. Brown                                    **
**                                                                           **
**      Modifications by many others; see the documentation for a list       **
**                                                                           **
**      Copyright 1994, Harry Flowers and Foteos Macrides                    **
**                                                                           **
**      Terms of Copyright:  This software may be freely distributed         **
**          and used by anyone, providing the distributor provides it        **
**          free of charge or at a price which only covers distribution      **
**          costs.  It may not be sold.  Note that versions prior to         **
**          V3.7.4 were public domain, and are not subject to these terms.   **
**                                                                           **
**      Disclaimer: No warranties as to the use or reliability of this       **
**          software are given by the authors or distributors.  Use at       **
**          your own risk.                                                   **
**                                                                           **
*******************************************************************************
******************************************************************************/

/*
**  swing_29 send a file to a queue
*/
#include <stdio.h>
#include <ssdef.h>
#include <smgdef.h>
#include <smgmsg.h>
#include <sjcdef.h>
#include <stsdef>
#include "more_descrip.h"
#include "swing.h"
#include "swing_refs.h"

#ifdef _PRINT_MENU
#include "printmenu.h"
#else
#include <quidef.h>
#endif

/*
** The local (Rutgers) fixes to suit the local setup
*/
#if defined(LOCALFIX)
#include "localfix.h"
#endif


/* Check if queue_name is a printer queue. Queues of type PRINTER,  */
/* TERMINAL and SERVER are all considered to be printer queues.     */

printer_queue(char *queue_name)
{
    int stat;
    struct queue_flags   qflags; 
    struct queue_status  qstatus;

    struct itemlist {
        short buffer_length;
        short item_code;
        char *buffer_address;
        short *return_length_address;
	} itmlst[] = {strlen (queue_name), QUI$_SEARCH_NAME, queue_name, 0,
		      4, QUI$_QUEUE_FLAGS, &qflags, 0,
		      4, QUI$_QUEUE_STATUS, &qstatus, 0,
		      0, 0, 0, 0};

 
    stat = sys$getquiw (0,
		        QUI$_DISPLAY_QUEUE,
		        0,
		        itmlst,
		        0, 0, 0, 0);
  
    if ((stat & 1) != 1) return TRUE;

    return qflags.qui$v_queue_printer  | 
           qflags.qui$v_queue_terminal |
           qstatus.qui$v_queue_server ; 

}

sndjbc_files(option,dir)
int option;
char *dir;
{
    /*
    **  option can be either PRINT_FILES or SUBMIT_FILES
    **  dir is current directory
    */
    FILER *temp;
    int status;
    long iosb[2];
    int notify_user;
    char new_queue[SPEC_SIZE+1];

    int printed = FALSE;

#if defined (LOCALFIX)
    char bin_number[SPEC_SIZE+1];
    char new_bin[SPEC_SIZE+1];
#endif

    struct itemlist {
        short buffer_length;
        short item_code;
        char *buffer_address;
        short *return_length_address;
	} itmlst[6];

    struct itemlist *itm = itmlst;

        if (option == PRINT_FILES)
        {
	    if (files_selected == 0 )
		sprintf(junk,"Print the current file Y/N [N]");
	    else
                 if (files_selected == 1)
                sprintf(junk,"Print the selected file Y/N [N]");
            else
                sprintf(junk,"Print the %d selected files Y/N [N]",files_selected);
        }
        else
        {
            if (files_selected == 1)
                sprintf(junk,"Submit/Noprint the selected file Y/N [N]");
            else
                sprintf(junk,"Submit/Noprint the %d selected files Y/N [N]",files_selected);
        }

        if (!confirm_user(junk,"Y"))
           return(TRUE);
    /*
    **  we'll only set up the default queue name first time around - there after
    **  we'll remember what the user selects
    */
    if (option == PRINT_FILES)
    {
        if (!print_queue[0])
        {
            strcpy(print_queue,"SYS$PRINT");
        }
    }
    else
    {
        if (!batch_queue[0])
        {
            strcpy(batch_queue,"SYS$BATCH");
        }
    }

    if (option == PRINT_FILES)
    {
#if defined (LOCALFIX)
      sprintf(junk,"%s",sys_getenv(PRINTQUE_LOGNM)!=NULL ?
	      sys_getenv(PRINTQUE_LOGNM) : PRINTQUE_DEF);
      strcpy(print_queue,junk);
      sprintf(junk,"%s",sys_getenv(BIN_LOGNM)!=NULL ?
	      sys_getenv(BIN_LOGNM) : BIN_NUMBER_DEF);
      strcpy (bin_number, junk );
      sprintf(junk,"Print with print/que=%s/note=%s ?",print_queue,bin_number);
      if (!confirm_user(junk,"Y"))
      {
#endif
#ifdef _PRINT_MENU
        sprintf(junk,"Printer Queue (? for list) [%s] : ",print_queue);
        if (!get_user_input(junk,new_queue,SPEC_SIZE))
	    if (print_queue[0] == '?')
	    {
	    	status = print_menu(new_queue,"SYS$PRINT");
		if (status == 1)  /*  Print Cancelled */
		{
		     message("Print Cancelled");
		     return(FALSE);
		}
		strcpy(print_queue,new_queue);
	    }
	    else
	    {
		strcpy(new_queue,print_queue);
	    }
        else {
	    status = 0;  /*  Initialize */
	    if (new_queue[0] == '?')
	    {
	    	status = print_menu(new_queue,print_queue);
		if (status == 1)  /*  Print Cancelled */
		{
		     message("Print Cancelled");
		     return(FALSE);
		}
	    }
	    strcpy(print_queue,new_queue);
	}
#else
        sprintf(junk,"Printer Queue [%s] ?",print_queue);
        if (!get_user_input(junk,new_queue,SPEC_SIZE))
            strcpy(new_queue,print_queue);
        else
            strcpy(print_queue,new_queue);
#endif
#if defined (LOCALFIX)
	sprintf(junk,"Bin-number [%s] ?",bin_number);
	if (!get_user_input(junk,new_bin,SPEC_SIZE))
	    strcpy(new_bin,bin_number);
	else
	    strcpy(bin_number,new_bin);
      }
      else
      {
	strcpy(new_bin,bin_number);
	strcpy(new_queue,print_queue);
      }
#endif
      if (!printer_queue(new_queue))
	{
	     message("Not a printer queue - print cancelled");
	     return(FALSE);
	}
    }
    else
    {
        sprintf(junk,"Batch Queue [%s] ?",batch_queue);
        if (!get_user_input(junk,new_queue,SPEC_SIZE))
            strcpy(new_queue,batch_queue);
        else
            strcpy(batch_queue,new_queue);
    }

    if (confirm_user("Notify user when completed Y/N [N] ?","Y"))
        notify_user = TRUE;
    else
        notify_user = FALSE;

    if (!files_selected)
	temp = current_file_ptr(curr_file_no);
    else
	temp = first_file_ptr;

    please_wait();

    while (temp && !printed)
    {
	if (!files_selected)
		printed = TRUE;
        if (temp->rendition || !files_selected)
        {

            /*
            **  o.k. - print this file to queue new_queue
            */
            itm->buffer_length = strlen(temp->filename);
            itm->item_code = SJC$_FILE_SPECIFICATION;
            itm->buffer_address = temp->filename;
            itm->return_length_address = NULL;
	    itm++;

            itm->buffer_length = strlen(new_queue);
            itm->item_code = SJC$_QUEUE;
            itm->buffer_address = new_queue;
            itm->return_length_address = 0;
	    itm++;

            if (notify_user)
	      {
		itm->buffer_length = 0;
                itm->item_code = SJC$_NOTIFY;
		itm->buffer_address = 0;
		itm->return_length_address = 0;
		itm++;
	      }

            /*
            **  make sure we get no log file spooling
            */

            if (option == SUBMIT_FILES)
	      {
		itm->buffer_length = 0;
		itm->item_code = SJC$_NO_LOG_SPOOL;
		itm->buffer_address = 0;
		itm->return_length_address = 0;
		itm++;

		itm->buffer_length = strlen(dir);
                itm->item_code = SJC$_LOG_SPECIFICATION;
		itm->buffer_address = dir;
		itm->return_length_address = 0;
		itm++;
	      }


#if defined (LOCALFIX)
            itm->buffer_length = strlen(new_bin);
            if (option == PRINT_FILES)
                itm->item_code = SJC$_NOTE;
            else
                itm->item_code = SJC$_NO_LOG_SPECIFICATION;
            itm->buffer_address = new_bin;
            itm->return_length_address = 0;
	    itm++;
#endif
            itm->item_code = 0;

            status = sys$sndjbcw(0,
                                 SJC$_ENTER_FILE,
                                 0,
                                 itmlst,
                                 &iosb[0],0,0);

            if (status != SS$_NORMAL             ||
                (iosb[0] & 0x7) == STS$K_ERROR   ||
                (iosb[0] & 0x7) == STS$K_SEVERE
               )
            {
                if (status == SS$_NORMAL)  status = iosb[0];

                sprintf(junk,"%s, FILE %s aborted",
                        full_error_text(status),temp->filename);
                message(junk);
            }
/***
            if (status != SS$_NORMAL || (iosb[0] & 0xffff) != SS$_NORMAL)
            {
                if (status == SS$_NORMAL)
                    status = iosb[0];

                sprintf(junk,"%s - %s - aborted",
                        error_text(status),temp->filename,status);
                message(junk);
                return(TRUE);
            }
***/
            else
            {
                if ((iosb[0] & 0x7) == STS$K_WARNING ||
                    (iosb[0] & 0x7) == STS$K_INFO
                   )
                {
                    sprintf(junk,"%s, FILE %s",
                        full_error_text(iosb[0]),temp->filename);
                    message(junk);
                }
                if (option == PRINT_FILES)
                    sprintf(junk,"%s has been queued for printing",
                            temp->filename);
                else
                    sprintf(junk,"%s has been submitted",temp->filename);

                message(junk);
            }
/***
            {
                if (option == PRINT_FILES)
                    sprintf(junk,"%s has been queued for printing",temp->filename);
                else
                    sprintf(junk,"%s has been submitted",temp->filename);

                message(junk);
            }
***/
        }
	if (files_selected) {
           temp = temp->forw_ptr;
	   itm = itmlst;
	}
    }
    return(TRUE);
}

#ifdef _PRINT_MENU
int 
print_menu(char *queue, char *print_queue) {
    DESC_ARRAY_1DIM(menu_desc,UNKNOWN,UNKNOWN,UNKNOWN);
    struct printqueue printers[MAX_PRINT_QUEUES];
    unsigned long status;
    FILER *file_ptr;
    char printdesc[MAX_PRINT_QUEUES][76], *quespec, *filename, *ext, *semi, *temp;
    int i,j,printcnt=0, curstart=0, extlen = 0;
    unsigned long menu_length, displayp, selection = 0;

    quespec = calloc(10,1);

    if (!files_selected)  /*  If no files selected, go based on current */
    {
	file_ptr = current_file_ptr(curr_file_no);
	filename = file_ptr->filename;
    }
    else
    {
	file_ptr = first_file_ptr;
	filename = file_ptr->filename;
	while (!file_ptr->rendition)  /*  Check for first selected file */
	{
	    file_ptr=file_ptr->forw_ptr;
	    filename=file_ptr->filename;
	}
    }
    temp = strrchr(filename,'.') + 1;  /*  Find . and then skip it */
    semi = strrchr(filename,';');
    extlen = semi - temp;
    ext = calloc(extlen+1,1);
    strncpy(ext,temp,extlen);
    ext[extlen] = 0;

    strcpy(quespec,"~");
    for (i=0; i<NUM_EXTS; i++)
    {
	if ((strcmp(ext,extmappings[i].extension)) == 0)
	{
	    strcpy(quespec,extmappings[i].designation);
	}
    }
    
    please_wait();  /*  Put "Please Wait..." on screen while getting queues */

    status = get_queues(printers, quespec, &printcnt);

    if (status != SS$_NORMAL)
    {
    	get_user_input("Unable to retrieve queue information - Hit <RETURN> to continue",
	    junk, 
	    SPEC_SIZE);
	sprintf(junk,"Printer Queue [%s] ?",print_queue);
	if (!get_user_input(junk,queue,SPEC_SIZE))
	{
	    strcpy(queue,print_queue);
	}
	return(0);
    }

    for (i=0;  i<MAX_PRINT_QUEUES;  i++)
    {
    	for (j=0;  j<76;  j++)
	{
	    printdesc[i][j] = 0;
	}
	if (i < printcnt) {
	    if (printers[i].status[0] == '*')
	    {
		 sprintf(printdesc[i],
		    "%-15s: %-50s %3d jobs",  /*  Change if Macros change - BW */
		    printers[i].name,
		    printers[i].desc,
		    printers[i].jobs);
	    }
	    else
	    {
		 sprintf(printdesc[i],
		    "%-15s: %-50s %-8s",  /*  Change if Macros change - BW */
		    printers[i].name,
		    printers[i].desc,
		    printers[i].status);
	    }
	}
    }

	    
    sprintf(printdesc[printcnt],"Exit without printing");

    if (printcnt == 0) 
    {
	get_user_input("Queue Descriptions not set, Unable to provide menu",
	    junk,
	    SPEC_SIZE);
	sprintf(junk,"Printer Queue [%s] ?",print_queue);
	if (!get_user_input(junk,queue,SPEC_SIZE))
	{
	    strcpy(queue,print_queue);
	}
    }
    else  /*  Pop up menu of printer queues */
    {
	message("Select the printer of your choice");
    	menu_length = page_length - 3;

	i = SMG$CREATE_VIRTUAL_DISPLAY (
	    	 &menu_length, 
	    	 &PRINT_MENU_WIDTH, 
	    	 &displayp, 
	    	 &SMG$M_BORDER);

	if (i != SS$_NORMAL)
	{
	    panic(i,"create virtual display displayp","print_menu");
	}

	  
	i = SMG$PASTE_VIRTUAL_DISPLAY (
	      &displayp, 
	      &paste, 
	      &2, 
	      &2);
  
	  
        /*	  
	**  SMG$LABEL_BORDER (
	**       &displayp, 
	**       " QUEUE NAME           QUEUE DESCRIPTION                             STATUS  ", 
	**       SMG$K_TOP);
	**   
	*/	  



	menu_desc.dsc$w_length = 76;
	menu_desc.dsc$a_pointer = printdesc;
	menu_desc.dsc$a_a0 = printdesc;
	menu_desc.dsc$l_arsize = 76 * MAX_PRINT_QUEUES;
	menu_desc.dsc$bounds[0].dsc$l_u = MAX_PRINT_QUEUES - 1;

	i = SMG$CREATE_MENU (
	    &displayp, 
	    &menu_desc, 
	    &SMG$K_VERTICAL, 
	    &SMG$M_FIXED_FORMAT);

	i = SMG$SELECT_FROM_MENU (
	    &keyboard, 
	    &displayp, 
	    &selection,
	    &0);
            /*	  
	    **  &0,
	    **	&SMG$M_RETURN_IMMED);
	    */	  

	i = SMG$DELETE_MENU(&displayp);

	i = SMG$DELETE_VIRTUAL_DISPLAY(&displayp);

	if (selection >= printcnt)
	{
	    return(1);  /*  Exit without printing */
	}
	    
	strncpy(queue,printers[selection].name,strlen(printers[selection].name));

	queue[strlen(printers[selection].name)] = 0;  /*  0-terminate it */

    }
    return(0);
}

int
get_queues(struct printqueue printers[MAX_PRINT_QUEUES], char *quespec, 
	int *printcnt)
{
    struct {
	    short buffer_length;
	    short item_code;
	    long int buffer_address;
	    long int return_length_address;
    } itmlst[10];

    char *quename, *quedesc, quefilter[2], *desctrim;
    long i, namelen, desclen, quejobs, questatus, iosb[2], status, hdrlen;

    quename = calloc(31,1);
    quedesc = calloc(255,1);
    strcpy(quefilter,"*");
    *printcnt = 0;

    itmlst[0].item_code = QUI$_SEARCH_NAME;  /*  Wildcard search for all queues */
    itmlst[0].buffer_length = 1;
    itmlst[0].buffer_address = quefilter;
    itmlst[0].return_length_address = &i;
    i = 0;

    itmlst[1].item_code = QUI$_QUEUE_NAME;  /*  Get Queue Name */
    itmlst[1].buffer_length = 31; 
    itmlst[1].buffer_address = quename;
    itmlst[1].return_length_address = &namelen;

    itmlst[2].item_code = QUI$_QUEUE_DESCRIPTION;  /*  Get Queue Description */
    itmlst[2].buffer_length = 255;
    itmlst[2].buffer_address = quedesc;
    itmlst[2].return_length_address = &desclen;

    itmlst[3].item_code = QUI$_EXECUTING_JOB_COUNT;  /*  Get # of jobs */
    itmlst[3].buffer_length = 4;
    itmlst[3].buffer_address = &quejobs;
    itmlst[3].return_length_address = &i;
    i = 0;

    itmlst[4].item_code = QUI$_QUEUE_STATUS;  /*  Get Queue Status */
    itmlst[4].buffer_length = 4;
    itmlst[4].buffer_address = &questatus;
    itmlst[4].return_length_address = &i;
    i = 0;

    itmlst[5].item_code = 0;  /*  Terminates item list */

    status = sys$getquiw (  /*  Get Next Queue in list */
	     	  0, 
	     	  QUI$_DISPLAY_QUEUE, 
	     	  0, 
	     	  itmlst, 
	     	  iosb, 
	     	  0, 
	     	  0);
    while ((iosb[0] != JBC$_NOMOREQUE) && (iosb[0] != JBC$_NOSUCHQUE)
	    && (status == SS$_NORMAL))
    {
    	if (strstr(quedesc,quespec))  /*  Description contains quespec */
	{
	    if (namelen > QUE_NAME_LEN)
	    {
	    	 strncpy(printers[*printcnt].name,quename,QUE_NAME_LEN);
		 printers[*printcnt].name[QUE_NAME_LEN]=0;
	    }
	    else
	    {
		strncpy(printers[*printcnt].name,quename,namelen);
		printers[*printcnt].name[namelen]=0;  /*  Null Terminate it */
	    }

	    desctrim = strchr(quedesc, QUE_DESC_DELIM); /* remove format list */
	    desctrim++;
	    hdrlen = desctrim - quedesc;	    

	    if ((desclen-hdrlen) > QUE_DESC_LEN)
	    {
	    	 strncpy(printers[*printcnt].desc,desctrim,QUE_DESC_LEN);
		 printers[*printcnt].desc[QUE_DESC_LEN] = 0;
	    }
	    else
	    {
	    	 strncpy(printers[*printcnt].desc,desctrim,desclen);
		 printers[*printcnt].desc[desclen-hdrlen] = 0;
	    }

	    printers[*printcnt].jobs = quejobs;
	    
	    if (questatus & QUI$M_QUEUE_UNAVAILABLE)  /*  Insert Appropriate status */
	    {
	    	 strcpy(printers[*printcnt].status,"UnAvail");
	    }
	    else if (questatus & QUI$M_QUEUE_PAUSED)
	    {
	    	 strcpy(printers[*printcnt].status,"Paused");
	    }
	    else if (questatus & QUI$M_QUEUE_PAUSING)
	    {
	    	 strcpy(printers[*printcnt].status,"Pausing");
	    }
	    else if (questatus & QUI$M_QUEUE_RESUMING)
	    {
	    	 strcpy(printers[*printcnt].status,"Resuming");
	    }
	    else if (questatus & QUI$M_QUEUE_STALLED)
	    {
	    	 strcpy(printers[*printcnt].status,"Stalled");
	    }
	    else if (questatus & QUI$M_QUEUE_STARTING)
	    {
	    	 strcpy(printers[*printcnt].status,"Starting");
	    }
	    else if (questatus & QUI$M_QUEUE_STOPPED)
	    {
	    	 strcpy(printers[*printcnt].status,"Stopped");
	    }
	    else if (questatus & QUI$M_QUEUE_STOPPING)
	    {
	    	 strcpy(printers[*printcnt].status,"Stopping");
	    }
	    else
	    {
		 strcpy(printers[*printcnt].status,"*");
	    }
	    (*printcnt)++;
	}
	quename[0]=0;
	quedesc[0]=0;
	questatus=0;
	quejobs=0;
	status = sys$getquiw (  /*  Get Next Queue in list */
	     	  0, 
	     	  QUI$_DISPLAY_QUEUE, 
	     	  0, 
	     	  itmlst, 
	     	  iosb, 
	     	  0, 
	     	  0);
    }
    return(status);
}
#endif
