/******************************************************************************
*******************************************************************************
**                                                                           **
**      C Swing - A graphical file and directory maintenance utility         **
**                                                                           **
*******************************************************************************
**                                                                           **
**      Original C coding by: S. J. Brown                                    **
**                                                                           **
**      Modifications by many others; see the documentation for a list       **
**                                                                           **
**      Copyright 1994, Harry Flowers and Foteos Macrides                    **
**                                                                           **
**      Terms of Copyright:  This software may be freely distributed         **
**          and used by anyone, providing the distributor provides it        **
**          free of charge or at a price which only covers distribution      **
**          costs.  It may not be sold.  Note that versions prior to         **
**          V3.7.4 were public domain, and are not subject to these terms.   **
**                                                                           **
**      Disclaimer: No warranties as to the use or reliability of this       **
**          software are given by the authors or distributors.  Use at       **
**          your own risk.                                                   **
**                                                                           **
*******************************************************************************
******************************************************************************/
 
/*
#define debug 		1
*/

/*
**  *** Beginning of Site Customizations ***
**
**  (Just move the #defines outside the comment lines to select that option.)
*/

/* define this to get the local (Rutgers) changes; you'll also need to compile
** and link with this as the command files as supplied do not use this option.
** Peter Chen 11/2/90
#define LOCALFIX 1
*/

/* Define this if you want [Return] to be a synonym for Exit in the Directory
** manager screen (rather than a synonym for F to enter the File mananger
** screen), and for Delete instead of [Return] to be the command for accepting
** the currently highlighted directory in copy, move, export and import
** maneuvers.  If defined, it includes CR_RETURNS_FROM_FILE_TO_DIR_MGR.
** Mats Akerberg: 06-May-91
#define MATS_HACK
*/

/* Define this if you want to use [Return] as a synonym for Quit to return
** from the File to Directory manager screen, but do not want the other
** components of the MATS_HACK.  If this but not MATS_HACK is defined,
** [Return] acts as a toggle between the File and Directory manager screens.
** If neither this nor MATS_HACK is defined, [Return] is a dead key in the
** File manager screen, and invokes the File manager screen in the Directory
** Manager screen.  (It will be defined below if MATS_HACK was defined above)
** Chris Butler: 11-Jul-91
#define CR_RETURNS_FROM_FILE_TO_DIR_MGR
*/

/* Define this if you want the Quit option in the Directory manager screen to
** exit into the original directory instead of the current one.  This restores
** the behavior of an older patch.  The Exit option and ^Z synonym always exit
** with the current directory as the default. 
#define Q_EXITS_TO_ORIGINAL_DIR
*/

/* Define this if you want to seek and use a device__000000__SWSVE.DAT file
** if a device__directory__SWSVE.DAT file was not found.  Defining it sets
** the intended behavior of v3.6.2 (with some bug fixes).  The consequence
** of using a __000000__ file is that a tree for the *entire* device is
** displayed.  I personally do not recommend defining it, because on occasions
** when you really do want the entire device tree displayed it's easy enough
** to include 000000 in the directory field of the command line argument.
** Moreover, I've added a command line qualifier so that if you do not
** define it, you can evoke this behavior by entering a -0 or /0 argument
** in the call to CSWING.					FM: 11-Sep-92
#define ALWAYS_TRY_000000
*/

/* Define this if you want to make SYS$SYSROOT the default device when you
** exit in a SYS$SPECIFIC tree and such (a) tree(s) had been put up via an 
** argument or logical translation that had SYS$SYSROOT as the device.  Note
** that this behavior will apply for all SYS$SPECIFIC trees if any have been
** put up via a SYS$$SYSROOT argument or translation, but not if all were
** put up via SYS$SPECIFIC:[directory] arguments.  If you do not define it,
** you can evoke this behavior by entering a -s or /s argument in the call
** to CSWING.							FM: 11-Sep-92
#define SPECIFIC_EXITS_TO_SYSROOT
*/

/* Define this if you want to make SYS$SYSROOT the default device when you
** exit in a SYS$COMMON tree and such (a) tree(s) had been put up via an 
** argument or logical translation that had SYS$SYSROOT as the device.  Note
** that this behavior will apply for all SYS$COMMON trees if any have been
** put up via a SYS$$SYSROOT argument or translation, but not if all were
** put up via SYS$COMMON:[directory] arguments.  If you do not define it,
** you can evoke this behavior by entering a -c or /c argument in the call
** to CSWING.							FM: 11-Sep-92
#define COMMON_EXITS_TO_SYSROOT
*/

/*
**  _PRINT_MENU enables the displaying of a menu of printers when the user
**  types '?' from the printer queue prompt.  See PRINTMENU.README for details
**  email:  Butch-Walker@uokhsc.edu
#define _PRINT_MENU
*/

/*
**  CHANGE_PROT_TO_DELETE is defined by default and will attempt to change the
**  file protection to allow a file to be deleted.  Sites which wish to honor
**  existing protection settings to make it harder to delete files may wish
**  to comment this definition out.
*/
#define CHANGE_PROT_TO_DELETE

/*
**  *** End of Site Customizations ***
*/

#ifdef MATS_HACK
#define CR_RETURNS_FROM_FILE_TO_DIR_MGR
#endif

/*
**  VAXC speed me ups
*/
#ifndef __DECC
#define malloc	VAXC$MALLOC_OPT
#define calloc  VAXC$CALLOC_OPT
#define free	VAXC$FREE_OPT
#define cfree	VAXC$CFREE_OPT
#define realloc	VAXC$REALLOC_OPT
#endif

#define VERSION	"C SWING v3.7.6 (S.J.Brown, M.Akerberg, H.Flowers, F.Macrides & others)"
#define DEFAULT_DIR_SPEC	"*.*"

#define AUTO_CONTINUAL_UPDATE	5

#define NO_TERMINATE_CHAR	'^'
#define NO_FILE_CHAR		'!'

#define CAPTIVE_ERRMSG		"Function disabled for captive accounts"

#define PRESS_TO_CONTINUE	"Press any key to continue..."
#define ESCAPE			27

#define LISTING_FILE		"SWING_LISTING_FILE"

#define SCROLL_WIDTH		75
#define MAX_FILES_SCROLL	14

#define WORLD_READ_WRITE	6

#define FM_TYPE_MAX		9

#define MOVE_FILES	1
#define COPY_FILES	2

#define SHOW_FILES		1
#define SHOW_DIRS		2
#define SHOW_FILES_AND_DIRS	3

#define PRINT_FILES	1
#define SUBMIT_FILES	2

#define INCLUDE_FILES	1
#define EXCLUDE_FILES	2

#define OL_FM_TYPE		0
#define OL_PRINT_QUEUE		1
#define OL_BATCH_QUEUE		2
#define OL_EDITOR_DEFN		3

#define FILE_MANAGER		0
#define USER_OPTIONS		1

#define AUTO_UPDATE_DELAY	5
#define MIN_BROADCAST_TIME	2
#define ALLOW_CLOCK		2
#define ALLOW_AUTO		1
#define NO_AUTO			0

#define CTRLY		0x02000000
#define CTRLT		0x00100000

#define IMPORT 100
#define EXPORT 101

#define DELETE 101
#define PURGE  202

#define SCREEN_WIDTH 	80
#define PAGE_MIN_LENGTH	24

#define NAME_SIZE       88
#define SPEC_SIZE 	255
#define MAX_LEVEL 	8
#define MAX_ADD_LEVEL   7
#define NAME_SHORT_SIZE 9
#define NAME_LONG_SIZE  15
#define BLANK_SHORT_NAME "         "

#define LEVEL_OFFSET(A) 	(A*(NAME_SHORT_SIZE+1))+1
#define C_DIAMOND	1
#define C_UP 		1
#define C_DOWN		2
#define C_LEFT		4
#define C_RIGHT		8

#define DISPLAY_DCL_ROWS	page_length
#define DISPLAY_DCL_COLUMNS	80

#define MESSAGE_ROWS		18
#define MESSAGE_COLS		78
#define MESSAGE_X		2
#define MESSAGE_Y		4

#define DISPLAY1_ROWS		(page_length-3)
#define DISPLAY1_COLUMNS	80
#define DISPLAY1_X              1
#define DISPLAY1_Y              3

#define DISPLAYF_ROWS		(page_length-5)
#define DISPLAYF_COLUMNS	78
/*
**  put the display over in the left
*/
#define DISPLAYF_X              2
#define DISPLAYF_Y              4

#define DISPLAYF_FILES		(DISPLAYF_ROWS-2)
#define CHOSEN_FILE_RENDITION	(SMG$M_REVERSE)
#define CURSOR_RENDITION 	(0)
#define FILE_PAGE_ROWS		(DISPLAYF_FILES-3)
#define LEFTF_COL		3
#define RIGHTF_COL              48

#define FILENAME_OFFSET		3
#define	CURSOR_SIZE		2

#define CURS_MIN		4
#define CURS_MAX                (DISPLAY1_ROWS-5)
#define MAX_B4_SCROLL		10
/* 
** for next-page and prev-page 
*/
#define PAGE_SIZE               DISPLAY1_ROWS

#define DISPLAYT_ROWS		2
#define DISPLAYT_COLUMNS	80
#define DISPLAYT_X              1
#define DISPLAYT_Y              1

#define STATUS_X		1
#define STATUS_Y		2

#define DISPLAYB_ROWS		1
#define DISPLAYB_COLUMNS	80
#define DISPLAYB_X              1
#define DISPLAYB_Y              page_length

#define CUR_REST_Y		1
#define CUR_REST_X		80

#define DELETE_SCREEN_ROWS	10
#define DELETE_SCREEN_COLUMNS	60

#define DELETE_MSG_Y		DELETE_SCREEN_ROWS
#define DELETE_MSG_X		2

#define DELETE_SCREEN_X		((SCREEN_WIDTH - DELETE_SCREEN_COLUMNS)/2 + 1)
#define DELETE_SCREEN_Y		7

#define DELETE_FILE_X		2
#define DELETE_FILE_Y		2

#define DELETE_STAT_X		2
#define DELETE_STAT_Y		4

#define DELETE_STAT1_X		2
#define DELETE_STAT1_Y		5

#define DELETE_STATUS_X		2
#define DELETE_STATUS_Y		7

#define DELETE_STATUS1_X	2
#define DELETE_STATUS1_Y	8

#define DELETE_STATUS2_X	2
#define DELETE_STATUS2_Y	9

#define _STR(A)		&A[strlen(A)]

#define SPECIAL_RENDITION	(SMG$M_UNDERLINE|SMG$M_BOLD)
#define NORMAL_RENDITION	(SMG$M_REVERSE)
#define UNDERLINE_RENDITION	(SMG$M_REVERSE | SMG$M_UNDERLINE)
#define DELETING_RENDITION      (SMG$M_BLINK)

#define CTITLE "Create    Quit     [RETURN]     Top      Bottom      Next page     Previous page!"
#define MTITLE "Create    Rename    Delete    Move    pUrge    Quit   Help    Files   Locate   ?!"
#define DTITLE "Quit    ^D = user definable options    Help    ? = quick help screen     $ = DCL!"
#define FTITLE "Edit   eXecute   Copy   Rename   Delete    Move   pUrge   Quit   Help   List   ?!"
#define UTITLE "Change-value    Restore-original-value   Exit(& save changes)   Quit    Help   ?!"
#define STITLE "Alloc  Count  Dir-name  Files  Grand-total  Idx  List  Max  Quit  Stats  Used  ?!"

#define SAVE_FILE_SPEC		"SWSVE.DAT;1"
#define USER_OPTIONS_FILE	"SWING_OPTIONS.INITIAL"

typedef unsigned long	ULONG;
typedef unsigned short	USHORT;

#ifdef __DECC				/* primarily for AXP */
#pragma member_alignment save
#pragma nomember_alignment
#endif

typedef struct _iosb {			/* I/O status block		      */
	 USHORT	  status,		/* status value			      */
		  count,		/* byte transfer count		      */
		  dummy[2];		/* not used			      */
} IOSB_DEF;

typedef struct _itmlst {
	unsigned short buffer_length;
	unsigned short item_code;
	void           *buffer_address;
	unsigned short *return_length_address;
} ITM_LST;

#ifdef __DECC				/* primarily for AXP */
#pragma member_alignment restore
#endif

/*
**  for holding display line structures
*/
typedef struct n_node {
char dir_name[SPEC_SIZE+1];
char name[NAME_SIZE+1];
char short_name[NAME_SHORT_SIZE+1];
char last_filename[NAME_SIZE+1];
char dir_spec[NAME_SIZE+1];
short in_or_ex_clude;
/*
**  did of this directory
*/
USHORT did[3];
/*
**  fid of this directory
*/
USHORT fid[3];
struct n_node *child;
struct n_node *sister;
struct n_node *lost;
unsigned long checksum;
int line;
int level;
/*
**  statistics
*/
int stats_total_files;
int stats_total_size_alloc;
int stats_total_size_used;
int stats_biggest_file;
int stats_non_zero_files;
int stats_total_pointers;
int stats_collected;
} NODE;

/*
**  for holding a directory structure header
*/
typedef struct dstructure {
NODE *fnp;
char device[NAME_SIZE];
char save_file_away[SPEC_SIZE+1];
char save_file_home[SPEC_SIZE+1];
char save_file_ss[SPEC_SIZE+1];
char dir_name[SPEC_SIZE+1];
char name[NAME_SIZE+1];
struct dstructure *forw_ptr;
int to_be_saved;
} DSTRUCTURE;

/*
**  for holding single directory structures
*/
typedef struct l_line {
struct l_line *forw_ptr;
struct l_line *back_ptr;
struct n_node *n_ptr[MAX_LEVEL+1];
struct dstructure *dp;
char connections[MAX_LEVEL+1];
} LINE;


#define CREDATE_SIZE 8

/*
**  for holding a file structure - NOTE, when adding more elements to 
**  this structure, also update reasses_files, where selected elements
**  are copied from one structure to another
*/
typedef struct fl {
/*
**  links to other structures
*/
struct fl *forw_ptr;
struct fl *back_ptr;

/*
**  filename
*/
char filename[NAME_SIZE+1];

/*
**  creation date
*/
unsigned char credate[CREDATE_SIZE+1];

unsigned char  file_org,	/* file organisation */
               rec_attr,	/* record attributes */
	       bkt,		/* bucket size */
               rec_type;	/* record type */

unsigned short did[3];		/* directory id */
unsigned short fid[3],		/* file id */
               placement,	/* placement information */
               grp,		/* group id */
               mbm,		/* member id */
               fpro,		/* file protection */
               defext,		/* default extension */
               gbc,		/* global buffer count */
               versions,	/* max number of versions for directory */
               verlim,	        /* max number of versions for file */
               rsize,		/* record size */
               maxrec;		/* maximum record size */

unsigned long  headers, 	/* file headers */
               fragments,	/* file fragments */
               size,		/* size */
               allocation,	/* allocation */
               uchar,		/* file characteristics */
               uic;		/* uic */

int info_lookup;		/* status from directory lookup */

int rendition;			/* current rendition */

} FILER;

typedef struct acb_struct {
        short  atr$w_size;
        short  atr$w_type;
        void   *atr$l_addr;
} ACB;

typedef struct fatdef_struct {
        char fat$b_rtype;
        char fat$b_rattrib;
        unsigned short fat$w_rsize;
        unsigned short fat$w_hiblk[2];
        unsigned short fat$w_efblk[2];
        unsigned short fat$w_ffbyte;
        char fat$b_bktsize;
	char fat$b_vfcsize;
	unsigned short fat$w_maxrec;
	unsigned short fat$w_defext;
	unsigned short fat$w_gbc;
        char _reserved[6];
        char _not_used[2];
	unsigned short fat$w_versions;
        unsigned long fat$l_sbn;
} FAT;

typedef struct user_opt_struct {
	int key;
	int no_terminate;
        char key_name[60];
        char value[SCREEN_WIDTH+1];
        char original[SCREEN_WIDTH+1];
        char *default_value;
} UOPT;

#ifdef __DECC

int	add_a_dstructure(char *specification);
int	add_directory_structure(char *directory);
int	add_dirs(void);
int	add_dstructure(char *specification);
void	add_sister_bars(LINE *ptr, int level);
void	big_display_off(void);
void	big_display_on(void);
int	big_message_init(void);
void	broadcasts_ready(void);
ULONG	change_the_ownership(USHORT channel, USHORT fid[3], ULONG owner);
ULONG	change_the_protection(USHORT channel, USHORT fid[3], USHORT protection,
                              USHORT *original);
ULONG	check_termtype(void);
int	clear_cursor(void);
int	confirm_user(char *text, char *check);
void	construct_file_specs(char *home, char *away, char *ss);
void	cont_update(void);
void	copy_window(int remaining, char *filename, char *device);
void	copy_window_centre(void);
void	copy_window_off(int prompt);
void	copy_window_percent(char *text, ULONG made, ULONG total);
int	create_listing(int option, int stats);
FILER	*current_file_ptr(int no);
LINE	*current_line_ptr(int line);
int	define_menu_options(void);
void	defragment_dirs(NODE *n);
int	defragment_directory(NODE *n);
ULONG	delete_a_file(char *spec);
int	delete_a_file_by_fid(USHORT *fid, USHORT *did, char *device);
int	delete_dirs(int option);
int	delete_many_files(NODE *n, int sub_dirs, int option, char *spec);
void	delete_screen_off(void);
void	delete_screen_on(char *text);
void	delete_screen_message(char *text);
void	delete_screen_status(char *filename, int ok, int not_ok, int tot, 
                                             int opt, char *text);
void	ding(ULONG display);
void	dir_stats(NODE *n);
void	disable_broadcasts(void);
void	disable_ctrl(void);
void	display_line(LINE *ptr, int line, int scroll, int direction);
int	edit_file(char *dir, char *filename);
void	enable_broadcasts(void);
void	enable_ctrl(void);
int	ends_in_excl(char *string);
char	*error_text(ULONG msgid);
int	execute_cmd(char *cmd, char *filename, int terminate);
int	execute_dcl(char *filename);
int	extract_structure(DSTRUCTURE *p);
void	file_manager_label(int file_type, NODE *node);
int	files_menu_options(int file_type);
void	files_so_far(long x);
int	find_next_node(NODE *n);
int	find_prev_node(NODE *n);
void	first_node_in_stack(NODE *n);
void	free_files(FILER *ptr);
void	free_lines(LINE *lptr);
int	free_nodes(NODE *n_ptr);
char	*full_error_text(ULONG msgid);
int	function_key(int key, char *filename);
ULONG	get_checksum(NODE *n);
int	get_did(char *dir, USHORT *did);
int	get_dirs(NODE *n_ptr, ULONG channel);
int	get_dir_files(NODE *n, int file_type);
int	get_dir_info(FILER *f);
int	get_key(int auto_update);
int	get_key_ast(void);
int	get_new_dir_spec(NODE *n);
void	get_root_dir(char *from, char *dir, char *name);
void	get_time(char *the_time);
int	get_user_input(char *text, char *string, size_t size);
int	get_user_input_with_prompt(char *text, char *string, size_t size);
ULONG	get_user_quota(char *device, ULONG *usage, ULONG *permquota, 
                       ULONG *overdraft);
void	give_node_a_name(NODE *p, char *name);
void	go_bottom(void);
void	go_down(void);
void	go_left(void);
void	go_next(void);
void	go_prev(void);
void	go_right(void);
void	go_top(void);
void	go_up(void);
FILER	*grab_file(char *filename);
LINE	*grab_line(void);
NODE	*grab_node(char *name);
void	label_copy_window(char *device);
ULONG	lib_copy_file(char *from, char *to, char *filename, 
                      int remaining, int option, int prot);
int	load_definitions(int option);
int	load_lines(void);
void	load_line(LINE *l_ptr, int line, int level, NODE *n_ptr, DSTRUCTURE *dp);
int	load_nodes_from_file(DSTRUCTURE *d);
void	load_up_node_names(DSTRUCTURE *d);
int	locate_file(void);
int	main_menu_options(void);
void	message(char *text);
void	modify_pbd(void);
int	most_files(NODE *n);
void	most_less(int num_most_args, char *most_args[]);
void	move_dir(void);
int	mung_files(NODE *n);
void	new_short_name(NODE *p, char *name);
int	next_command(int option, int move_or_copy);
void	no_directories(void);
void	no_files(NODE *node);
int	panic(int code, char *text, char *where);
int	parse_file(char *filename);
void	please_wait(void);
void	prot_to_string(USHORT prot, char *string);
void	protect_dirs(int option);
int	protect_files(NODE *n, int file_type, int option);
void	purge_dirs(void);
int	put_scroll_line(int line, FILER *fptr, int display_type);
int	put_sub_title(char *text, int effect);
void	*read_a_directory(NODE *n, long *blocks);
int	redo_display(int option);
void	refresh_display(void);
int	remote_files(char *dir, int file_count, int option);
void	rename_dir(void);
void	rest_cursor(void);
ULONG	lib_rename(char *from, char *to, char *name);
int	remove_visited_dir(NODE *n);
void	render(int line,int level, int rendition);
void	repair_display(void);
void	rest_cursor(void);
int	save_node(NODE *nptr, FILE *fp);
int	save_nodes(DSTRUCTURE *d, int verbal);
int	search_nodes(char *dir, char *name, DSTRUCTURE *dp);
void	set_cursor(int line);
int	set_default(char *dir);
void	set_init_position(void);
void	set_protection(char *device, USHORT fid[3], USHORT protection,
                       USHORT *original);
void	show_broadcasts(void);
void	show_def(char *output);
void	show_deleting(NODE *nptr, int rendition, int sisters);
void	show_deleting_1(NODE *nptr, int rendition, int sisters);
void	show_files(int changed, int display_type);
void	show_quotas(NODE *n);
void	show_version(void);
int	snapshot(void);
int	sndjbc_files(int option, char *dir);
void	spawn_process(char *command, char *prompt);
void	spawn_the_process(char *command, char *prompt);
int	stats_menu(void);
void	structure_inconsistent(char *dir);
void	sub_title_normal(char *text);
void	sub_title_underline(char *text);
void	swing_files(NODE *n, char *filename, int file_type);
void	swing_help(char *start);
void	title(char *text);
ULONG	translate(char *string, char *res);
void	travel_to_dir(void);
void	travel_to_dir_remote(void);
void	update_clock(void);
void	user_options(int option, char *filename, NODE *n, int file_type);
void	warning(char *text);
void	warning_no_files(NODE *node);

#endif

