/******************************************************************************
*******************************************************************************
**                                                                           **
**      C Swing - A graphical file and directory maintenance utility         **
**                                                                           **
*******************************************************************************
**                                                                           **
**      Original C coding by: S. J. Brown                                    **
**                                                                           **
**      Modifications by many others; see the documentation for a list       **
**                                                                           **
**      Copyright 1994, Harry Flowers and Foteos Macrides                    **
**                                                                           **
**      Terms of Copyright:  This software may be freely distributed         **
**          and used by anyone, providing the distributor provides it        **
**          free of charge or at a price which only covers distribution      **
**          costs.  It may not be sold.  Note that versions prior to         **
**          V3.7.4 were public domain, and are not subject to these terms.   **
**                                                                           **
**      Disclaimer: No warranties as to the use or reliability of this       **
**          software are given by the authors or distributors.  Use at       **
**          your own risk.                                                   **
**                                                                           **
*******************************************************************************
******************************************************************************/
 
/*
**  swing_10 spawned processes
*/
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <descrip.h>
#include <smgdef.h>
#include <smgmsg.h>
#include <smg$routines.h>
#include <rms.h>
#include <ssdef.h>
#include <unistd.h>
#include <starlet.h>
#include <lib$routines.h>
#include <str$routines.h>
#include "more_descrip.h"
#include "swing.h"
#include "swing_refs.h"

#define BANNER "------------------------------ SWING DCL SUBPROCESS --------------------------------"
#define GIVE_ME_A_KEY 	"Press any key "
#define PROMPT       	"SWING > "


int ends_in_excl(char *string)
{
    int i,l;
 
    /*
    **  if string is terminated by ! plus any number of spaces return the
    **  position of the !, else return FALSE
    */
    l = strlen(string) - 1;

    for (i = l; i >= 0; i--)
    {
        if (string[i] == '!')
            return(i);
        else if (string[i] != ' ')
            return(FALSE);
    }

    /*
    **  it's just spaces, no use to man or dog, so return FALSE
    */
    return(FALSE);
}

int execute_cmd(char *cmd, char *filename, int terminate)
{
    long length, screen;
    char indata[10];
    long col;
    int just_command, i;
    char command[SPEC_SIZE+1];

    /*
    **  check user really wants to do this
    **
    **  if the last char of command is a ! we'll not add the filename
    **  and we'll drop the !
    */

    strcpy(command,cmd);
    
    i = ends_in_excl(command);
    if (i)
    {
        just_command = TRUE;
        command[i] = 0;
        sprintf(junk,"$ %s ?Y/N [N]",command);
    }
    else
    {
        just_command = FALSE;
        sprintf(junk,"$ %s %s ?Y/N [N]",command,filename);
        if (strlen(junk) > 79)
        {
            sprintf(junk,"$ %s %s ?Y/N [N]",command,"<file>");
            if (strlen(junk) > 79)
            {
                sprintf(junk,"$ %s ?Y/N [N]",command);
                if (strlen(junk) > 79)
                {
                    sprintf(junk,"$%.70s~ ?Y/N [N]",command);
                }
            }
        }
    }

    /*
    **  if we don't want to do this return FALSE
    */
    if (!terminate && !confirm_user(junk,"Y"))
        return(FALSE);

    /*
    **  set up display
    */
    smg$save_physical_screen(&paste,&screen);
    smg$erase_display(&display_dcl);
    smg$put_line(&display_dcl,DSC1(BANNER),&1,0);
    smg$paste_virtual_display(&display_dcl,&paste,&1,&1);

    if (just_command)
        sprintf(junk,"$ %s",command);
    else
        sprintf(junk,"$ %s %s",command,filename);

    smg$put_line(&display_dcl,DSC1(junk),&1,0);
    spawn_process(junk,0);

    /*
    **  get user to hit return to continue unless no terminate is set
    */
    if (!terminate)
    {
        col = SCREEN_WIDTH - strlen(GIVE_ME_A_KEY);
        smg$set_cursor_abs(&display_dcl,&DISPLAY_DCL_ROWS,&col);
        smg$read_string(&keyboard,
                        DSCA1(indata),
                        DSC1(GIVE_ME_A_KEY),
                        &1,0,0,0,&length,0,&display_dcl,0,0);
    }

    /*
    **  get the terminal width. if not what we set it to originally
    **  then set it back
    */
    smg$erase_display(&display_dcl);
    modify_pbd();
    smg$unpaste_virtual_display(&display_dcl, &paste);
    smg$restore_physical_screen(&paste,&screen);

    /*
    **  we ran all the way through
    */
    return(TRUE);
}

int execute_dcl(char *filename)
{
    long screen,status,length,i,index,sub_index;
    char indata[10];
    int col;

    if (filename[0])
    {
        sprintf(junk,"Really execute %s ?Y/N [N]",filename);
        if (!confirm_user(junk,"Y"))
            return(TRUE);
    }

    /*
    **  the name of the file to be executed is in filename
    */

    /*
    **  set up display
    */
    smg$save_physical_screen(&paste,&screen);
    smg$erase_display(&display_dcl);
    smg$put_line(&display_dcl,DSC1(BANNER),&1,0);

    if (!filename[0])
    {
        sprintf(junk,"Logout when finished");
        smg$put_line(&display_dcl,DSC1(junk),&2,0);
    }
    smg$paste_virtual_display(&display_dcl,&paste,&1,&1);

    if (filename[0])
    {
        /*
        **  if .exe we run it, else we @ it
        */
        status = str$find_first_substring(DSC1(filename), 
                      &index,       
                      &sub_index, 
                      DSC2(".EXE;"));
        if((status & 1) == 1)
        {
	    strcpy(junk, "$");
	    show_def(junk+1);
            strcat(junk,filename);
            lib$set_symbol(DSC1("foreign_command_temp"),DSC2(junk),&2);
            sprintf(junk,"$ foreign_command_temp");
        }
        else
        {
            status = str$find_first_substring(DSC1(filename), 
                           &index,       
                           &sub_index, 
                           DSC2(".COM;"));
            if((status && 1) == 1)
            {
                sprintf(junk,"$ @%s",filename);
            }
        }
        if((status & 1) != 1)
        {
            sprintf(junk,"Execute %s as a command file ?Y/N [N]",filename);
        
            indata[0] = 0;
    
            /*
            **  get user to hit return to continue
            */
            smg$read_string(&keyboard,
                            DSCA2(indata),
                            DSC1(junk),
                            &1,0,0,0,&length,0,&display_dcl,0,0);
    
            if (toupper (indata[0]) == 'Y')
            {
                sprintf(junk,"$ @%s",filename);
            }
            else
                goto way_out;
        }
    
        smg$put_line(&display_dcl,DSC1(junk),&1,0);
        spawn_process(junk,0);
        /*
        **  get user to hit return to continue
        */
        col = SCREEN_WIDTH - strlen(GIVE_ME_A_KEY);
        smg$set_cursor_abs(&display_dcl,&DISPLAY_DCL_ROWS,&col);
        smg$read_string(&keyboard,
                        DSCA1(indata),
                        DSC1(GIVE_ME_A_KEY),
                        &1,0,0,0,&length,0,&display_dcl,0,0);
    }
    else
    {
        /*
        **  now spawn a subprocess specifying DCL as the CLI
        */
        spawn_process(0,PROMPT);
    }

way_out:
    /*
    **  get the terminal width. if not what we set it to originally
    **  then set it back
    */
    smg$erase_display(&display_dcl);
    modify_pbd();
    smg$unpaste_virtual_display(&display_dcl, &paste);
    smg$restore_physical_screen(&paste,&screen);
}

void disable_broadcasts(void)
{
    ULONG  status;

    status = smg$disable_broadcast_trapping(&paste);
    if((status & 1) != 1)
        panic(status, "disable broadcast trapping","disable broadcasts");
}

void enable_broadcasts(void)
{
    ULONG  status;

    status = smg$set_broadcast_trapping(&paste,broadcasts_ready);
    if((status & 1) != 1)
        panic(status,"set broadcast trapping","enable broadcasts");
}

void spawn_process(char *command, char *prompt)
{
    ULONG  status;
    ULONG  mask;

    mask = CTRLT + CTRLY;
    /*
    **  now spawn a subprocess to do this command
    */
    disable_broadcasts();
    spawn_the_process(command,prompt);
    enable_broadcasts();
}

ULONG spawn_exception(int *sigarr, int *mecharr)
{
    /*
    **  unwind and return
    */
    lib$sig_to_ret(sigarr, mecharr);

    return(SS$_UNWIND);
}

void spawn_the_process(char *command, char *prompt)
{
    ULONG  status;
    int    proc_id;
    globalref int captive;

    VAXC$ESTABLISH(spawn_exception);

    proc_id = 0;

    enable_ctrl();

    if (command)
    {
        status = lib$spawn(DSC1(command), 0, 0, 0, 0,&proc_id, 
                           0, 0, 0, 0, 0);
    }
    else
    {
	if (captive)
	{
	    warning(CAPTIVE_ERRMSG);
	}
	else
	{
            status = lib$spawn(0, 0, 0, 0, 0, &proc_id, 
                               0, 0, 0, 0, DSC1(prompt));
    
            if((status & 1) != 1)
                warning("can't spawn a child process");
        }
    }

    disable_ctrl();

    /*
    **  slaughter child if left running
    **  check proc_id not my pid, also that it's not 0, 
    **  which is also me
    */
    if (proc_id && proc_id != getpid())
        sys$delprc(&proc_id,0);
}
