/******************************************************************************
*******************************************************************************
**                                                                           **
**      C Swing - A graphical file and directory maintenance utility         **
**                                                                           **
*******************************************************************************
**                                                                           **
**      Original C coding by: S. J. Brown                                    **
**                                                                           **
**      Modifications by many others; see the documentation for a list       **
**                                                                           **
**      Copyright 1994, Harry Flowers and Foteos Macrides                    **
**                                                                           **
**      Terms of Copyright:  This software may be freely distributed         **
**          and used by anyone, providing the distributor provides it        **
**          free of charge or at a price which only covers distribution      **
**          costs.  It may not be sold.  Note that versions prior to         **
**          V3.7.4 were public domain, and are not subject to these terms.   **
**                                                                           **
**      Disclaimer: No warranties as to the use or reliability of this       **
**          software are given by the authors or distributors.  Use at       **
**          your own risk.                                                   **
**                                                                           **
*******************************************************************************
******************************************************************************/
 
/* 
**  swing_31 file manager
*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <iodef.h>
#include <ssdef.h>
#include <strdef.h>
#include <fchdef.h>
#include <atrdef.h>
#include <fibdef.h>
#include "fatdef.h"
#include "fm2def.h"
#include "fh2def.h"
#include <starlet.h>
#include <lib$routines.h>
#include <str$routines.h>
#include "more_descrip.h"
#include "swing.h"
#include "swing_refs.h"

#define FILES_INFO_SIZE 100

/*
**  acb elements
*/
unsigned long uchar;
char credate[ATR$S_CREDATE+1];
long uic;
short fpro;
unsigned char header[ATR$S_HEADER+1];

/*
**  record attribute block
*/
FAT fat;

/*
**  acb descriptor for io$_access
*/
static
ACB acb2[] =   {
               /*
               **  file characteristics (directory ??)
               */
               ATR$S_UCHAR,     ATR$C_UCHAR,    &uchar,

               /*
               **  creation date
               */
               ATR$S_CREDATE,	ATR$C_CREDATE,	credate,

               /*
               **  protection
               */
               ATR$S_FPRO,	ATR$C_FPRO,	&fpro,

    	       /*
               **  file owner
               */
               ATR$S_UIC,	ATR$C_UIC,	&uic,               

               /*
               **  record attributes
               */
               ATR$S_RECATTR,	ATR$C_RECATTR,	&fat,

               /*
               **  file header for map area information
               */
               ATR$S_HEADER,	ATR$C_HEADER,	&header[0],

               /*
               **  terminate the list
               */
               0,               0,               0};

/*
**  acb descriptor for uchar
*/
ACB acb_uchar[] =   {
               /*
               **  file characteristics (directory ??)
               */
               ATR$S_UCHAR,     ATR$C_UCHAR,    &uchar,

               /*
               **  terminate the list
               */
               0,               0,               0};

/*
**  just the header
*/
ACB acb_hdr[] = {
               /*
               **  file header for map area information
               */
               ATR$S_HEADER,	ATR$C_HEADER,	&header[0],

               /*
               **  terminate the list
               */
               0,               0,               0};


/*
**  a blank fib used for initialising other fibs
*/
struct fibdef blank_fib, fib;

static int check_if_dir(char *filename, USHORT fid[3]);
static int map_area(FILER *f);


int get_dir_files(NODE *n, int file_type)
{
    ULONG  status;
    long   blocks;
    int str_status;
    int i, b, j;
    int name;
    unsigned short *buffer;
    unsigned short version;
    unsigned short fid[3];
    short count;
    short vers;
    short len;
    char new_file[SPEC_SIZE+1];
    char tmp[NAME_SIZE+1];

    /*
    **  max_owner_width = 0
    */
    max_owner_width = 0;
    buffer = (USHORT *) read_a_directory(n,&blocks);
    if (buffer == NULL)
        return(FALSE);

    /*
    **  work out the partial match string we'll use
    */
    strcpy(tmp,n->dir_spec);
    strcat(tmp,";*");

    /*
    **  now process the directory
    */
    for (b = 0; b < blocks; b++)
    {
        /*
        **  block at a time
        */
        for (i = b * 256; i < (b+1) * 256; )
        {
            /*
            **  pull out records
            */
            if (buffer[i] == 0xffff)
            {
                /*
                **  end of data for this block
                */
                break;
            } 
            else
            {
                /*
                **  we've a record
                */
                count = buffer[i];
                len = buffer[i+2] >> 8;
                name = i+3;
                if (len % 2)
                {
                    vers = (count - 5 - len)/8;
                    i += (4 + len/2);
                }
                else
                {
                    vers = (count - 4 - len)/8;
                    i += (3 + len/2);
                }
    
                for (j = 0; j < vers; j++)
                {
                    version = buffer[i];
                    fid[0] = buffer[i+1];
                    fid[1] = buffer[i+2];
                    fid[2] = buffer[i+3];
		    /*
		    ** Patch for volume sets
		    */
		    if ((fid[2] & 255) == 0)
			 fid[2] = (fid[2] | (n->fid[2] & 255));    
                    /*
                    **  advance ready for next info
                    */
                    i += 4;
                    sprintf(new_file,
                            "%.*s;%d",
                            len,
                            (char *) &buffer[name],
                            version);
                    /*
                    **  if we're looking for non-directories and this is not a
                    **  directory or we're looking for directories and this is a
                    **  directory we're in and we match the search spec
                    */
                    if (file_type == SHOW_FILES)
                        str_status = str$match_wild(DSC1(new_file),
                                                    DSC2(tmp));

                    if ( file_type == SHOW_FILES_AND_DIRS ||
			 ( file_type == SHOW_FILES && 
                           ( ( str_status == STR$_MATCH &&
			       n->in_or_ex_clude == INCLUDE_FILES ) ||
			     ( str_status != STR$_MATCH &&
			       n->in_or_ex_clude == EXCLUDE_FILES ) ) && 
                           !check_if_dir(new_file,fid) ) ||
                         ( file_type == SHOW_DIRS  && 
                         check_if_dir(new_file,fid) ) )
                    {
                        /*
                        **  one more file found
                        */
                        grab_file(new_file);
                        curr_file_ptr->info_lookup = FALSE;
                        curr_file_ptr->fid[0] = fid[0];
                        curr_file_ptr->fid[1] = fid[1];
                        curr_file_ptr->fid[2] = fid[2];
                        curr_file_ptr->did[0] = n->fid[0];
                        curr_file_ptr->did[1] = n->fid[1];
                        curr_file_ptr->did[2] = n->fid[2];
        
                        if (!(total_files % FILES_INFO_SIZE))
                            files_so_far(total_files);
                    }
                }
            }
        }
    }
    
    /*
    **  free up out directory buffer
    */
    free(buffer);
    
    /*
    **  if we've said how many files there are we'll 
    **  make sure we leave the right count on the screen
    */
    if (total_files > FILES_INFO_SIZE)
        files_so_far(total_files);
}

static int check_if_dir(char *filename, USHORT fid[3])
{
    ULONG  status;
    IOSB_DEF iosb;
    int index, sub_index;
    struct dsc$descriptor_s fib_desc;

    /*
    **  do the simple test first - assume all dirs end .DIR;
    */
    status = str$find_first_substring(DSC1(filename),
                                      &index,
                                      &sub_index,
                                      DSC2(".DIR;"));
    if((status & 1) == 1)
    {
        /*
        **  we'll have to get the uchar to see if it's really a directory
        */

        fib = blank_fib;
        
        fib_desc.dsc$w_length  = FIB$C_LENGTH;
        fib_desc.dsc$a_pointer = (void *) &fib;
        fib_desc.dsc$b_class   = DSC$K_CLASS_S;
        fib_desc.dsc$b_dtype   = DSC$K_DTYPE_T;
        
#ifdef __DECC
        fib.fib$w_fid[0] = fid[0];
        fib.fib$w_fid[1] = fid[1];
        fib.fib$w_fid[2] = fid[2];
#else
        fib.fib$r_fid_overlay.fib$w_fid[0] = fid[0];
        fib.fib$r_fid_overlay.fib$w_fid[1] = fid[1];
        fib.fib$r_fid_overlay.fib$w_fid[2] = fid[2];
#endif

        status = sys$qiow(0,
                          dir_channel,
                          IO$_ACCESS,
                          &iosb,
                          0,
                          0,
                          &fib_desc,
                          0,
                          0,
                          0,
                          acb_uchar,
                          0);
        if((status & 1) == 1)			/* ## was != !! *
            status = iosb.status;

        /*
        **  if there is an error we'll ignore it and not stop the 
        **  processing
        */
        if((status & 1) != 1)
            return(FALSE);

        /*
        **  yeah, directory
        */
        if (uchar & FCH$M_DIRECTORY)
            return(TRUE);
    }
    return(FALSE);
}

int get_dir_info(FILER *f)
{
    ULONG  status;
    IOSB_DEF iosb;
    int i;
    static struct dsc$descriptor_s fname_desc;
    struct dsc$descriptor_s fib_desc;

    /*
    **  get the header info
    */

    fib = blank_fib;
    
    fname_desc.dsc$w_length  = strlen(f->filename);
    fname_desc.dsc$a_pointer = f->filename;
    fname_desc.dsc$b_class   = DSC$K_CLASS_S;
    fname_desc.dsc$b_dtype   = DSC$K_DTYPE_T;
    
    fib_desc.dsc$w_length  = FIB$C_LENGTH;
    fib_desc.dsc$a_pointer = (void *) &fib;
    fib_desc.dsc$b_class   = DSC$K_CLASS_S;
    fib_desc.dsc$b_dtype   = DSC$K_DTYPE_Z;
    
#ifdef __DECC
    fib.fib$w_fid[0] = f->fid[0];
    fib.fib$w_fid[1] = f->fid[1];
    fib.fib$w_fid[2] = f->fid[2];

    fib.fib$w_did[0] = f->did[0];
    fib.fib$w_did[1] = f->did[1];
    fib.fib$w_did[2] = f->did[2];
#else
    fib.fib$r_fid_overlay.fib$w_fid[0] = f->fid[0];
    fib.fib$r_fid_overlay.fib$w_fid[1] = f->fid[1];
    fib.fib$r_fid_overlay.fib$w_fid[2] = f->fid[2];

    fib.fib$r_did_overlay.fib$w_did[0] = f->did[0];
    fib.fib$r_did_overlay.fib$w_did[1] = f->did[1];
    fib.fib$r_did_overlay.fib$w_did[2] = f->did[2];
#endif

    status = sys$qiow(0,
                      dir_channel,
                      IO$_ACCESS,
                      &iosb,
                      0,
                      0,
                      &fib_desc,
                      &fname_desc,
                      0,
                      0,
                      acb2,
                      0);
    if((status & 1) != 1)
    {
        f->info_lookup = status;
        return(FALSE);
    }

    if((iosb.status & 1) != 1)
    {
        f->info_lookup = iosb.status;
        return(FALSE);
    }

    f->info_lookup = SS$_NORMAL;

    /*
    **  we got directory info
    */

    /*
    **  allocation
    */
    f->allocation =  (fat.fat$w_hiblk[1] & 0x0000ffff) + 
                    ((fat.fat$w_hiblk[0] << 16) & 0xffff0000);

    /*
    **  size - 
    **  if file is sequential we'll work out the size from eof
    **  and first free byte in eof block,
    **  otherwise we use highest allocated VBN
    */
    if ((fat.fat$b_rtype >> FAT$V_FILEORG) ==  FAT$C_SEQUENTIAL)
    {
        /*
        **  get eof block and ffbyte. if ffbyte == 0
        **  then we've used eof blocks - 1;
        */
        f->size =  (fat.fat$w_efblk[1] & 0x0000ffff) +
                  ((fat.fat$w_efblk[0] << 16) & 0xffff0000);

        if (fat.fat$w_ffbyte == 0)
        {
            f->size--;
        }
    }
    else
    {
        f->size = f->allocation;
    }
    
    /*
    **  do the placement control, count of headers, count of
    **  file fragments, placement information
    */
    map_area(f);

    /*
    **  owner, calculated from uic
    */
    f->grp = (short)(uic >> 16);
    f->mbm = (short)uic;
    f->uic = uic;

    for (i = 0; i < 8; i++)
        f->credate[i] = credate[i];

    f->fpro = fpro;

    /*
    **  file_organisation
    */
    
    f->file_org = (fat.fat$b_rtype >> FAT$V_FILEORG);
    f->uchar = uchar;
    f->defext = fat.fat$w_defext;
    f->gbc = fat.fat$w_gbc;
    f->bkt = fat.fat$b_bktsize;
    f->versions = fat.fat$w_versions;
    f->verlim = fib.fib$w_verlimit;

    /*
    **  record type
    */
    f->rec_type = fat.fat$b_rtype & 0x0f;
    f->rsize = fat.fat$w_rsize;
    f->maxrec = fat.fat$w_maxrec;

    /*
    **  record attributes
    */
    f->rec_attr = fat.fat$b_rattrib;
}

static int map_area(FILER *f)
{
    unsigned short *w, p;

    int start, in_use;
    int i, status;
    int file_accessed;
    IOSB_DEF iosb;
    unsigned short first_fid[3];
    unsigned short fid[3];
    struct dsc$descriptor_s fib_desc;

    f->headers = f->fragments = 0;
    f->placement = 0;
    first_fid[0] = f->fid[0];
    first_fid[1] = f->fid[1];
    first_fid[2] = f->fid[2];
    file_accessed = 0;

    /*
    **  if there are any extension headers we'll do a 
    **  IO$_ACCESS | IO$M_ACCESS on the file
    */
    if (header[FH2$W_EX_FIDNUM] || header[FH2$W_EX_FIDNUM+1])
    {
        /*
        **  access this file so that we can read in the headers
        */
        fib = blank_fib;

        fib_desc.dsc$w_length  = FIB$C_LENGTH;
        fib_desc.dsc$a_pointer = (void *) &fib;
        fib_desc.dsc$b_class   = DSC$K_CLASS_S;
        fib_desc.dsc$b_dtype   = DSC$K_DTYPE_T;

#ifdef __DECC
        fib.fib$w_fid[0] = first_fid[0];
        fib.fib$w_fid[1] = first_fid[1];
        fib.fib$w_fid[2] = first_fid[2];
#else
        fib.fib$r_fid_overlay.fib$w_fid[0] = first_fid[0];
        fib.fib$r_fid_overlay.fib$w_fid[1] = first_fid[1];
        fib.fib$r_fid_overlay.fib$w_fid[2] = first_fid[2];
#endif

        status = sys$qiow(0,
                          dir_channel,
                          IO$_ACCESS | IO$M_ACCESS,
                          &iosb,
                          0,
                          0,
                          &fib_desc,
                          0,
                          0,
                          0,
                          0,
                          0);

        if((status & 1) != 1)
        {
            warning(full_error_text(status));
            return(FALSE);
        }

        if((iosb.status & 1) != 1)
        {
            warning(full_error_text(iosb.status));
            return(FALSE);
        }
       
        file_accessed = TRUE;
    }
    /*
    **  make sure we're an ODS-2 disk
    */
    while(header[FH2$B_STRUCLEV] == 2)
    {
        f->headers++;
     
        start = (unsigned int)header[FH2$B_MPOFFSET] * 2;
        in_use = (unsigned int)header[FH2$B_MAP_INUSE] * 2;
    
        for (i = start; i < start + in_use; )
        {
            /*
            **  point in words
            */
            w = (USHORT *) &header[i];
            p = w[0] >> FM2$V_FORMAT;
    
            switch(p)
            {
        case FM2$C_PLACEMENT:
                f->placement = p;
                i += FM2$C_LENGTH0;
                break;
    
        case FM2$C_FORMAT1:
                i += FM2$C_LENGTH1;
                f->fragments++;
                break;
    
        case FM2$C_FORMAT2:
                i += FM2$C_LENGTH2;
                f->fragments++;
                break;
    
        case FM2$C_FORMAT3:
                 i += FM2$C_LENGTH3;
                f->fragments++;
                break;

        default:
                /*
                **  pointers have gone wrong - abandon ship
                */
                i += in_use;
                break;
            }
        }

        /*
        **  now we must look to see if there is another file header
        */
        w = (USHORT *) &header[FH2$W_EX_FIDNUM];
        fid[0] = w[0];
        w = (USHORT *) &header[FH2$W_EX_FIDSEQ];
        fid[1] = w[0];
        w = (USHORT *) &header[FH2$W_EX_FIDRVN];
        fid[2] = w[0];

        if (fid[0])
        {
            /*
            **  there is, so now we get the header again
            **  use the continuation file id
            */
            fib_desc.dsc$w_length  = FIB$C_LENGTH;
            fib_desc.dsc$a_pointer = (void *) &fib;
            fib_desc.dsc$b_class   = DSC$K_CLASS_S;
            fib_desc.dsc$b_dtype   = DSC$K_DTYPE_T;

            fib = blank_fib;
#ifdef __DECC
            fib.fib$w_fid[0] = fid[0];
            fib.fib$w_fid[1] = fid[1];
            fib.fib$w_fid[2] = fid[2];
#else
            fib.fib$r_fid_overlay.fib$w_fid[0] = fid[0];
            fib.fib$r_fid_overlay.fib$w_fid[1] = fid[1];
            fib.fib$r_fid_overlay.fib$w_fid[2] = fid[2];
#endif

            status = sys$qiow(0,
                              dir_channel,
                              IO$_ACCESS,
                              &iosb,
                              0,
                              0,
                              &fib_desc,
                              0,
                              0,
                              0,
                              acb_hdr,
                              0);

            if((status & 1) != 1)
            {
                warning(full_error_text(status));
                break; 
            }

            if((iosb.status & 1) != 1)
            {
                warning(full_error_text(iosb.status));
                break; 
            }
        } 
        else
            break;
    }

    /*
    **  if we accessed the file we'll deaccess it
    */
    if (!file_accessed)
        return(TRUE);

    fib = blank_fib;

    fib_desc.dsc$w_length  = FIB$C_LENGTH;
    fib_desc.dsc$a_pointer = (void *) &fib;
    fib_desc.dsc$b_class   = DSC$K_CLASS_S;
    fib_desc.dsc$b_dtype   = DSC$K_DTYPE_T;

#ifdef __DECC
    fib.fib$w_fid[0] = first_fid[0];
    fib.fib$w_fid[1] = first_fid[1];
    fib.fib$w_fid[2] = first_fid[2];
#else
    fib.fib$r_fid_overlay.fib$w_fid[0] = first_fid[0];
    fib.fib$r_fid_overlay.fib$w_fid[1] = first_fid[1];
    fib.fib$r_fid_overlay.fib$w_fid[2] = first_fid[2];
#endif

    status = sys$qiow(0,
                      dir_channel,
                      IO$_DEACCESS,
                      &iosb,
                      0,
                      0,
                      &fib_desc,
                      0,
                      0,
                      0,
                      0,
                      0);
    if((status & 1) != 1)
    {
        warning(full_error_text(status));
    } 
    else if((iosb.status & 1) != 1)
    {
        warning(full_error_text(iosb.status));
    }
}

void *read_a_directory(NODE *n, long *blocks)
{
    ULONG  status;
    USHORT channel;
    IOSB_DEF iosb;
    int my_blocks;
    int i, buff_size;
    char *buffer;
    struct dsc$descriptor_s fd;
    char device[SPEC_SIZE+1];
 
    strcpy(device,n->dir_name);

    for (i = 0; i < strlen(device); i++)
    {
        if (device[i] == ':')
        {
            device[i+1] = 0;
            break;
        }
    }

/* ###    channel = 0;	??? Obsolete - bad type */
    my_blocks = 0;
    buffer = NULL;

    status = sys$assign(DSC1(device),
                        &channel,
                        0,
                        0);
    if((status & 1) != 1)
    {
        warning(full_error_text(status));
        return(NULL);
    }

    fib = blank_fib;
    fd.dsc$w_length = FIB$C_LENGTH;
    fd.dsc$a_pointer = (void *) &fib;
    fd.dsc$b_class = DSC$K_CLASS_S;
    fd.dsc$b_dtype = DSC$K_DTYPE_T;

#ifdef __DECC
    fib.fib$w_fid[0] = n->fid[0];
    fib.fib$w_fid[1] = n->fid[1];
    fib.fib$w_fid[2] = n->fid[2];
#else
    fib.fib$r_fid_overlay.fib$w_fid[0] = n->fid[0];
    fib.fib$r_fid_overlay.fib$w_fid[1] = n->fid[1];
    fib.fib$r_fid_overlay.fib$w_fid[2] = n->fid[2];
#endif

    status = sys$qiow(0,
                      channel,
                      IO$_ACCESS | IO$M_ACCESS,
                      &iosb,
                      0,
                      0,
                      &fd,
                      0,
                      0,
                      0,
                      acb2,
                      0);
    if((status & 1) != 1)
    {
        warning(full_error_text(status));
    }
    else if((iosb.status & 1) != 1)
    {
        warning(full_error_text(iosb.status));
    }
    else
    {
        /*
        **  size - 
        **  if file is sequential we'll work out the size from eof
        **  and first free byte in eof block,
        **  otherwise we use highest allocated VBN
        */
        if ((fat.fat$b_rtype >> FAT$V_FILEORG) ==  FAT$C_SEQUENTIAL)
        {
            /*
            **  get eof block and ffbyte. if ffbyte == 0
            **  then we've used eof blocks - 1;
            */
            my_blocks = fat.fat$w_efblk[1] +
                                 (fat.fat$w_efblk[0] << 16);
    
            if (fat.fat$w_ffbyte == 0)
            {
                my_blocks--;
            }
        }
        else
        {
            /*
            **  allocation
            */
            my_blocks = fat.fat$w_hiblk[1] + (fat.fat$w_hiblk[0] << 16);
        }
        buff_size = my_blocks * 512;
        buffer = malloc(buff_size);

        status = sys$qiow(0,
                          channel,
                          IO$_READVBLK | IO$M_DATACHECK,
                          &iosb,
                          0,
                          0,
                          buffer,
                          buff_size,
                          1,
                          0,
                          0,
                          0);
        if((status & 1) != 1)
        {
            warning(full_error_text(status));
        }
        else if((iosb.status & 1) != 1)
        {
            warning(full_error_text(iosb.status));
        }
   
       status = sys$qiow(0,
                         channel,
                         IO$_DEACCESS,
                         &iosb,
                         0,
                         0,
                         &fd,
                         0,
                         0,
                         0,
                         0,
                         0);
       if((status & 1) != 1)
       {
           warning(full_error_text(status));
       } 
       else if((iosb.status & 1) != 1)
       {
           warning(full_error_text(iosb.status));
       }
    }

    status = sys$dassgn(channel);
    if((status & 1) != 1)
    {
        warning(full_error_text(status));
    }
    *blocks = my_blocks;
    return(buffer);
}
