/******************************************************************************
*******************************************************************************
**                                                                           **
**      C Swing - A graphical file and directory maintenance utility         **
**                                                                           **
*******************************************************************************
**                                                                           **
**      Original C coding by: S. J. Brown                                    **
**                                                                           **
**      Modifications by many others; see the documentation for a list       **
**                                                                           **
**      Copyright 1994, Harry Flowers and Foteos Macrides                    **
**                                                                           **
**      Terms of Copyright:  This software may be freely distributed         **
**          and used by anyone, providing the distributor provides it        **
**          free of charge or at a price which only covers distribution      **
**          costs.  It may not be sold.  Note that versions prior to         **
**          V3.7.4 were public domain, and are not subject to these terms.   **
**                                                                           **
**      Disclaimer: No warranties as to the use or reliability of this       **
**          software are given by the authors or distributors.  Use at       **
**          your own risk.                                                   **
**                                                                           **
*******************************************************************************
******************************************************************************/
 
/*
**  swing_9		HELP PROCESSING
*/
#include <stdio.h>
#include <string.h>
#include <hlpdef.h>
#include <ssdef.h>
#include <jpidef.h>
#include <smgdef.h>
#include <smgmsg.h>
#include <smg$routines.h>
#include <unistd.h>
#include <rms.h>
#include <starlet.h>
#include <lbr$routines.h>
#include <lib$routines.h>
#include "more_descrip.h"
#include "swing.h"
#include "swing_refs.h"

#define LBR$_ENDTOPIC 2525256

int  lines_output;
char input_present;
char input_string[132] = "";
USHORT  input_length;
ULONG  input_status;

$DESCRIPTOR(input_string_desc,input_string);
$DESCRIPTOR(string1_desc, " ");
$DESCRIPTOR(string2_desc, "Press RETURN to continue ... ");

static ULONG help_exception(int *sigarr, int *mecharr);
static ULONG help$get_input(struct dsc$descriptor *input, 
                            struct dsc$descriptor *prompt,
                            USHORT *x_input_length);
static ULONG help$put_output(struct dsc$descriptor *output);
static int   init_help(void);
static void  lbr_output_help(char *start);

ULONG lib$erase_page();			/* Very old?? library function? */



/*
**  we'll check for the presence of the help file in 
**
**	1) SWING$HLP
**	2) SYS$HELP
**	3) same dir as swing is executing from
*/
static int init_help(void)
{
    ITM_LST itmlst[2];

    ULONG  status;
    ULONG  pid;
    USHORT length;
    IOSB_DEF iosb;
    int i;
    char imagename[SPEC_SIZE + 1];


    if (help_initialized)
        return(TRUE);
    else
    {
        help_initialized = 1;
        /*
        **  note 0 ==> check if exists
        */
        help_spec[0] = 0;

        if (access("SWING$HLP:CSWING.HLB",0) == 0)
        {
            strcpy(help_spec,"SWING$HLP:CSWING.HLB");
            return(TRUE);
        }
        else if (access("SYS$HELP:CSWING.HLB",0) == 0)
        {
            strcpy(help_spec,"SYS$HELP:CSWING.HLB");
            return(TRUE);
        }
        else
        {
            /*
            **  we'll look in the same directory as this .exe file
            **
            **  use sys$getjpi to help us out
            */
            itmlst[0].buffer_length = SPEC_SIZE;
            itmlst[0].item_code = JPI$_IMAGNAME;
            itmlst[0].buffer_address = imagename;
            itmlst[0].return_length_address = &length;
            itmlst[1].buffer_length = itmlst[1].item_code = 0;

            length = 0;
            pid = getpid();
            status = sys$getjpiw(0,			/* efn */
                                 &pid,			/* pidadr */
                                 0,			/* prcnam */
                                 &itmlst,		/* itmlst */
                                 &iosb,			/* iosb */
                                 0,			/* astadr */
                                 0);			/* astprm */
            if((status & 1) != 1)
                panic(status,"getjpi","init help");
            
            /*
            **  ensure return string null terminated
            */
            imagename[length] = 0;                        

            /*
            **  look down spec for a ]
            */
            for (i = strlen(imagename); i > 0; i--)
            {
                if (imagename[i] == ']')
                {
                    imagename[i+1] = 0;
                    break;
                }
            }
            
            /*
            ** tag on help file name
            */
            strcat(imagename,"CSWING.HLB");
            if (access(imagename,0) == 0)
            {
                strcpy(help_spec,imagename);
                return(TRUE);
            }
        }
        return(FALSE);
    }
}

void swing_help(char *start)
{
    init_help();

    if (help_spec[0])
    {
        int save;

    	smg$save_physical_screen(&paste, &save);

        lbr_output_help(start);
        
        smg$restore_physical_screen(&paste, &save);
    }
    else
        warning("Help file CSWING.HLB not found in SWING$HLP, SYS$HELP or SWING image directory");
}

static ULONG help_exception(int *sigarr, int *mecharr)
{
    /*
    **  unwind and return
    */
    lib$sig_to_ret(sigarr, mecharr);

    return(SS$_UNWIND);
}

/*
**  output-routine for paged library display
**  substitute for lib$put_output in calls to lbr$output_help()
*/
static ULONG help$put_output(struct dsc$descriptor *output)
{
    int	status;

    /*
    **  if help page is not full, increment lines_output
    **  else, prompt for input from user, and if it's not
    **      a carriage_return, inform lbr$output_help()
    */
    if (lines_output < (page_length - 4))
	lines_output++;
    else
        if (lines_output == (page_length - 4)) {
	    status = lib$put_output (&string1_desc);
	    input_status = lib$get_input (&input_string_desc,
	    				  &string2_desc,
					  &input_length);
	    if ((input_length > 0) || (input_status == RMS$_EOF)) {
	        input_present = TRUE;
		return(LBR$_ENDTOPIC);
	    }
	    lines_output = page_length - 3;
	}

        /*
	**  see if we need to clear the screen
	*/
	if (lines_output > (page_length - 4)) {
            status = lib$erase_page(&1,&1);
            lines_output = 0;
	}

    status = lib$put_output(output);
    return(status);
}

/*
**  input-routine for paged library display
**  substitute for lib$get_input in calls to lbr$output_help()
*/
static ULONG help$get_input(struct dsc$descriptor *input, 
                            struct dsc$descriptor *prompt,
                            USHORT *x_input_length)
{
    /*
    **  if we already have input, use it
    **  else get input
    */
    if (input_present) {
    	strncpy(input->dsc$a_pointer,
		input_string_desc.dsc$a_pointer,
		input_length);
	input->dsc$w_length = input_length;
        input_present = FALSE;
    }
    else {
        input_status = lib$get_input(input,prompt,&input_length);
	input->dsc$w_length = input_length;
    }

    /*
    **  generate a page break after every input gotten
    */
    lines_output = page_length - 3;
    *x_input_length = input_length;

    return (input_status);
}

static void lbr_output_help(char *start)
{
    ULONG  status;
    int flags;

    /*
    **  the as necessary exception handler, cos lbr$output_help
    **  can crash if given a rubbish help file
    */
    VAXC$ESTABLISH(help_exception);

    /*
    **  initialize variables for input and output routines.
    */
    input_present = FALSE;
    flags = HLP$M_HELP | HLP$M_PROMPT | HLP$M_PAGE;
    lines_output = 0;

    /*
    **  display the help
    */
    status = lbr$output_help(&help$put_output,	/* output-routine */
                        &SCREEN_WIDTH,		/* output-width */
                        DSC1(start),		/* line-desc */
                        DSC2(help_spec),	/* library-name */
                        &flags,             	/* flags */
                        &help$get_input);	/* input-routine */
    if((status & 1) != 1)
        panic(status,"help","swing help");
}
