/******************************************************************************
*******************************************************************************
**                                                                           **
**      MOST for C Swing and Gopher - A file viewer                          **
**                                                                           **
*******************************************************************************
**                                                                           **
**      Original C coding by: John E. Davis                                  **
**                                                                           **
**      Modifications primarily by Foteos Macrides                           **
**                                                                           **
**      Copyright 1991, John E. Davis                                        **
**                                                                           **
**      Terms of Copyright: See MAIN.C                                       **
**                                                                           **
**      Disclaimer: No warranties as to the use or reliability of this       **
**          software are given by the authors or distributors.  Use at       **
**          your own risk.                                                   **
**                                                                           **
*******************************************************************************
******************************************************************************/
/*
 *  MOST:  sysdep.c
 *
 * HISTORY
 * {1}	19-Mar-91  Henk D. Davids <hdavids@mswe.dnet.ms.philips.nl>
 * 	History started. Added default input file name so you do not
 *	have to specify name or type if you want it to be *.
 * 	Changes indicated by "-hdd" in comment.
 * 
 *  2.  4/4/91  John E. Davis
 *      I added code to read the teminal size for unix systems-- at least it
 *      works on a sun4 (BSD ?).  In addition I have also recently added file
 *      deletion code for both unix and vms.
 *
**  Modified for C SWING and GOPHER compatibility by Foteos Macrides
**                                                MACRIDES@SCI.WFEB.EDU
*/

#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unixio.h>
#include <starlet.h>
#include <lib$routines.h>
#include "sysdep.h"
#include "externs.h"
#include "display.h"
#include "window.h"

#ifdef VMS

#ifdef __DECC				/* primarily for AXP */
#pragma member_alignment save
#pragma nomember_alignment
#endif

typedef struct _itmlst {
	unsigned short buffer_length;
	unsigned short item_code;
	void           *buffer_address;
	unsigned short *return_length_address;
} ITM_LST;

/*
** For Shell Commands
*/
unsigned long SHELL_PID = 0;  
#endif /* VMS */

/*
** For Terminal IO
*/
char INPUT_BUFFER[136];
int  INPUT_BUFFER_LEN = 0;
#ifdef VMS
int Have_MostTTY_chan   = 0;    /* flag that we have a TTY channel    */
int most_restored_width = 0;    /* flag that width restore performed  */
termchar MOST_START_TC;         /* starting terminal characteristics  */
termchar MOST_CURRENT_TC;       /* current terminal characteristics   */
int MOST_START_WS;              /* starting terminal wrap state       */
#endif


/* 
 *                          SHELL COMMANDS
 */
  
#ifdef VMS
/*
** these two from emacs source
*/  
void define_logical_name (char *varname, char *string)
{
    struct dsc$descriptor_s strdsc =
      {strlen (string), DSC$K_DTYPE_T, DSC$K_CLASS_S, string};
    struct dsc$descriptor_s envdsc =
      {strlen (varname), DSC$K_DTYPE_T, DSC$K_CLASS_S, varname};
    struct dsc$descriptor_s lnmdsc =
      {7, DSC$K_DTYPE_T, DSC$K_CLASS_S, "LNM$JOB"};

    lib$set_logical (&envdsc, &strdsc, &lnmdsc, 0, 0);
}

void delete_logical_name (char *varname)
{
    struct dsc$descriptor_s envdsc =
      {strlen (varname), DSC$K_DTYPE_T, DSC$K_CLASS_S, varname};
    struct dsc$descriptor_s lnmdsc =
      {7, DSC$K_DTYPE_T, DSC$K_CLASS_S, "LNM$JOB"};
    
    lib$delete_logical (&envdsc, &lnmdsc);
}
/* from emacs */

/*
** try to attach to a kept emacs editor
** returns 1 on success
*/
int do_emacs_command()
{
    unsigned long status;
    unsigned long pid;
    char *pidstr;
    
    if((pidstr = getenv("EMACS_PID")) != NULL)
      {
	  (void) sscanf(pidstr,"%X",&pid);
          /*
          ** we attach to emacs
          */
          status = lib$attach(&pid);
          if((status & 1) == 1)
	    {
               return(1);
	    }
          else
/** 	    printf("Can't attach to pid %X\n",pid); *//* we use most_message **/
            return(0);
      }
    else return(0);
}

/* 
** Code is here which could:
** try to attach to the parent if we're a subprocess
**   otherwise to a previous shell if one exists
**   else just spawn a new subprocess
** But instead we always spawn subprocess.
** Returns 1 on success
*/
int do_shell_command()
{
    int status;
    unsigned long parent_pid, completion_status;

    $DESCRIPTOR(MOST_$_descr, "MOST > ");

    /*
    ** see if there's a parent we could attach to
    *
    parent_pid = getppid();
    if (parent_pid && parent_pid != 0xffffffff)
      /*
      ** try to attach to parent
      *
      status = lib$attach(&parent_pid);

    /*
    ** see if there's a previous shell we could attach to
    *
    else if (SHELL_PID && SHELL_PID != 0xffffffff)
      /*
      ** try to attach to previous shell
      *
      status = lib$attach (&SHELL_PID);

    if((status & 1) != 1)
      /*
      ** others failed so spawn a new shell
      */
      {
	  fputs("Spawning MOST DCL SUBPROCESS (Logout when finished)...",
                stdout);
	  status = lib$spawn(0,0,0,0,0,&SHELL_PID,
                             &completion_status,0,0,0,&MOST_$_descr);
          if((status & 1) != 1)
            {
                most_message("Failed to spawn.",1);
                return(0);
            }
      }

    most_message(" ",0);  /* make sure message window is restored */
    return(1);
}
#endif /* VMS */

/*
** Spawn to execute a single system command
** returns 1 on success
*/
int do_most_system_command(char *command, int pause)
{
    char ch;
    if ((system(command) & 1) != 1)
      {
         most_message("Spawn to execute system command failed.",1);
         return(0);
      }

    if (pause)
      {
         fputs("\nPress RETURN to resume MOST",stdout);
         while ((ch = getkey()) != '\015' && ch != '\012') ;
      }
    most_message(" ",0);  /* make sure message window is restored */
    return(1);
}


/* 
 *                            FILE IO
 */

#ifdef VMS
/*
** convert user's filespec to full VMS filespec
**   returns 1 on success
** calling routine can reiterate until it gets a
**   0 returned, to deal with wildcards, and should do
**      'if (context) LIB$FIND_FILE_END(&context);'
**   when finished getting filespec(s)
*/
int expand_file_name(char *file, char *expanded_file, unsigned long *context)
{
    int status;
    unsigned long zero = 0;
    char name[255], ch;
    $DESCRIPTOR(result,name);
    struct dsc$descriptor_s  file_desc;
    int i;

    file_desc.dsc$w_length = strlen(file);
    file_desc.dsc$a_pointer = file;
    file_desc.dsc$b_class = DSC$K_CLASS_S; /* scalar, string type */
    file_desc.dsc$b_dtype = DSC$K_DTYPE_T; /* ascii string */

    status = 1;
    
    status = lib$find_file(&file_desc,&result,context,0,0,0,&zero);
    if((status & 1) == 1)
      {
          i = 0;
          while( ch = result.dsc$a_pointer[i], expanded_file[i++] = ch,
	         ch != ' ');
          expanded_file[--i] = '\0';
          return (1);
      }
    else
      {
          expanded_file[0] = '\0';       /* so file comes back as zero width */
          return(0);
      }
}
#endif /* VMS */


/* 
 *                          TERMINAL IO
 */


/*
** set up or restore terminal characteristics and use of keypad
**   when entering most
** returns 1 on success
*/
int init_most_tty()
{
#ifdef VMS
    unsigned long status;
    iosb iostatus;
    termchar tc;
    $DESCRIPTOR (MOST_Term, "SYS$ERROR");

    if (!Have_MostTTY_chan)
      /*
      ** this is the first call to init_most_tty()
      */
      {
        /*
        ** get a channel
        */
        status = sys$assign(&MOST_Term, &MOST_TTY_CHANNEL, 0, 0);
        if((status & 1) != 1)
	  {
	     fputs("Unable to assign input channel.\n", stderr);
	     /* exit(0); */ return(0);
	  }
	/*
	** get the starting terminal characteristics
	*/
        status = sys$qiow(              /* Queue and wait               */
                         0,             /* Wait on event flag zero      */
                      MOST_TTY_CHANNEL, /* Channel to input terminal    */
                         IO$_SENSEMODE, /* Get current characteristic   */
                         &iostatus,     /* Status after operation       */
                         0, 0,          /* No AST service               */
                     &MOST_START_TC,    /* Terminal characteristics buf */
               sizeof(MOST_START_TC),   /* Size of the buffer           */
                         0, 0, 0, 0);   /* P3-P6 unused                 */
        if((status & 1) != 1)
	  {
	     fputs("Unable to initialize input channel.\n", stderr);
	     return(0);
	  }

        /*
        ** save the starting wrap state and set nowrap
        */
        MOST_START_WS = (MOST_START_TC.t_mandl & 512);
        tc = MOST_START_TC;
        tc.t_mandl = (tc.t_mandl & (int)(-1 ^ 512));
        status = sys$qiow(                /* Queue and wait               */
                          0,              /* Wait on event flag zero      */
                     MOST_TTY_CHANNEL,    /* Channel to input terminal    */
                          IO$_SETMODE,    /* Get current characteristic   */
                          &iostatus,      /* Status after operation       */
                          0, 0,           /* No AST service               */
                          &tc,            /* Terminal characteristics buf */
                          sizeof(tc),     /* Size of the buffer           */
                          0, 0, 0, 0);    /* P3-P6 unused                 */
        if((status & 1) != 1)
	  {
	     fputs("Unable to set nowrap.\n", stderr);
	     return(0);
	  }

        /*
        ** set flag
        */
	Have_MostTTY_chan = 1;
      }
    else
      /*
      ** this is a subsequent call to init_most_tty()
      */
      {
         /*
         ** check the width and height vs the current tc for most
         **   restore them if changed, and
         **   switch physical terminal to narrow/wide mode
         **     if it was changed
         */
         if (!get_term_dimensions(&SCREEN_WIDTH, &SCREEN_HEIGHT))
           {
              most_message("Unable to get screen dimensions.", 1);
	      return(0);
           }
         if (SCREEN_WIDTH != (int) MOST_CURRENT_TC.t_width)
	   {
              SCREEN_WIDTH = (int) MOST_CURRENT_TC.t_width;
              SCREEN_WIDTH_M1 = SCREEN_WIDTH - 1;
	      if (SCREEN_WIDTH == 80)
                {
                   narrow_width();
                   most_restored_width = 1;
                }
              else if (SCREEN_WIDTH == 132)
                {
                   wide_width();
                   most_restored_width = 1;
                }
              else most_restored_width = 0;
           }
         SCREEN_HEIGHT = (int) ((unsigned int) MOST_CURRENT_TC.t_mandl >> 24);
         SCREEN_ROWS = SCREEN_HEIGHT - 1;

         /*
         ** restore the current terminal characteristics for most
         */
         status = sys$qiow(                /* Queue and wait               */
                           0,              /* Wait on event flag zero      */
                      MOST_TTY_CHANNEL,    /* Channel to input terminal    */
                           IO$_SETMODE,    /* Get current characteristic   */
                           &iostatus,      /* Status after operation       */
                           0, 0,           /* No AST service               */
                     &MOST_CURRENT_TC,     /* Terminal characteristics buf */
               sizeof(MOST_CURRENT_TC),    /* Size of the buffer           */
                           0, 0, 0, 0);    /* P3-P6 unused                 */
        if((status & 1) != 1)
	  {
	     most_message("Unable to restore terminal characteristics.\n", 1);
	     return(0);
	  }
      }
#else    
#ifdef BSD
    struct sgttyb  newtty;
    struct tchars tc;
    struct ltchars lc;

    ioctl(TTY_DESCR, TIOCGLTC,&OLDTTY_LC);
    ioctl(TTY_DESCR, TIOCGETC,&OLDTTY_TC);
    ioctl(TTY_DESCR, TIOCGETP, &OLDTTY);
    newtty = OLDTTY;
    newtty.sg_flags |= CBREAK;
    newtty.sg_flags &= ~(ECHO | XTABS);
    newtty.sg_erase = 0;
    newtty.sg_kill = 0;
    tc.t_intrc = 0;
    tc.t_quitc = 0;
    tc.t_eofc = 0;
    tc.t_brkc = 0;
    lc.t_suspc = 0;
    lc.t_dsuspc = 0;
    ioctl(TTY_DESCR, TIOCSETP, &newtty);
    ioctl(TTY_DESCR, TIOCSLTC,&lc);
    ioctl(TTY_DESCR, TIOCSETC,&tc);
#else /* other unix */    
    struct termio newtty;
	  
    ioctl(TTY_DESCR, TCGETA, &OLDTTY);
    ioctl(TTY_DESCR, TCGETA, &newtty);
    newtty.c_iflag &= ~(ECHO);  
    newtty.c_iflag &= ~(INLCR);  
    newtty.c_iflag &= ~(ICRNL);  
    newtty.c_lflag = 00000000; 
    newtty.c_cc[VMIN] = 1; 
    newtty.c_cc[VTIME] = 0;
    newtty.c_cc[VEOF] = 1;          
    ioctl(TTY_DESCR, TCSETA, &newtty);

#endif  /* other unix */
#endif

    /*
    ** enable the application keypad
    */
    fputs("\033=", stdout);

    return(1);
}

#ifndef VMS
sys_flush(int fd)
{
#ifdef BSD
    ioctl(fd, TIOCFLUSH, (char *) NULL);
#else
    ioctl(fd, TCFLSH, (char *) NULL);
#endif    
}
#endif    

/*
** restore starting tc for exits, or save current tc before spawns (VMS)
** (Unix: restore previous tc)
** returns 1 on success
*/
int reset_most_tty()
{
    globalref int exit_most;
#ifdef VMS
    int status;
    iosb iostatus;
    termchar tc;
#endif

    /* 
    ** make sure the application keypad is enabled
    */
    fputs("\033=", stdout);

#ifdef VMS
    if (!exit_most)
    /*
    ** save the current terminal characteristics
    */
      {
         status = sys$qiow(              /* Queue and wait               */
                          0,             /* Wait on event flag zero      */
                       MOST_TTY_CHANNEL, /* Channel to input terminal    */
                          IO$_SENSEMODE, /* Get current characteristic   */
                          &iostatus,     /* Status after operation       */
                          0, 0,          /* No AST service               */
                      &MOST_CURRENT_TC,  /* Terminal characteristics buf */
                sizeof(MOST_CURRENT_TC), /* Size of the buffer           */
                          0, 0, 0, 0);   /* P3-P6 unused                 */
         if((status & 1) != 1)
           {
              most_message("Unable to save current terminal characteristics.",
                           1);
              return(0);
           }
      }

    /*
    ** check the width and height vs the starting tc for most
    **   restore them if changed, and
    **   switch physical terminal to narrow/wide mode
    **     if it was changed
    */
    if (SCREEN_WIDTH != (int) MOST_START_TC.t_width)
      {
         SCREEN_WIDTH = (int) MOST_START_TC.t_width;
         SCREEN_WIDTH_M1 = SCREEN_WIDTH - 1;
         if (SCREEN_WIDTH == 80)
           {
              narrow_width();
              most_restored_width = 1;
           }
         else if (SCREEN_WIDTH == 132)
           {
              wide_width();
              most_restored_width = 1;
           }
         else most_restored_width = 0;
      }
    SCREEN_HEIGHT = (int) ((unsigned int) MOST_START_TC.t_mandl >> 24);
    SCREEN_ROWS = SCREEN_HEIGHT - 1;
    
    /*
    ** restore the starting tc
    */
    status = sys$qiow(                /* Queue and wait               */
                      0,              /* Wait on event flag zero      */
                 MOST_TTY_CHANNEL,    /* Channel to input terminal    */
                      IO$_SETMODE,    /* Get current characteristic   */
                      &iostatus,      /* Status after operation       */
                      0, 0,           /* No AST service               */
                &MOST_START_TC,       /* Terminal characteristics buf */
          sizeof(MOST_START_TC),      /* Size of the buffer           */
                      0, 0, 0, 0);    /* P3-P6 unused                 */
    if((status & 1) != 1)
      {
           if (exit_most)
             {
                fputs("Unable to restore starting terminal characteristics.",
                       stderr);
	        return(0);
             }
           else
             {
	        most_message("Unable to restore wrap state.", 1);
	        return(0);
             }
      }

    if (exit_most)
      {
         Have_MostTTY_chan = 0;
         if((sys$dassgn(MOST_TTY_CHANNEL)  & 1) == 0)
              return(0);
      }
#endif /* VMS */
#ifndef VMS
#ifdef BSD
	ioctl(TTY_DESCR, TIOCSETP, &OLDTTY);
	ioctl(TTY_DESCR, TIOCSLTC, &OLDTTY_LC);
	ioctl(TTY_DESCR, TIOCSETC, &OLDTTY_TC);
#else        
    	ioctl(TTY_DESCR, TCSETA, &OLDTTY);
#endif /* BSD */
#endif /* not VMS */

    return(1);
}

/*
** character fetch routines
*/
char sys_getkey()
{
    char c;
    
#ifdef VMS
    int status;
    static int trmmsk [2] = { 0, 0 };
    iosb iostatus;

    status = sys$qiow ( 0, MOST_TTY_CHANNEL,
                       IO$_READVBLK + IO$M_NOECHO | IO$_TTYREADALL,
                       &iostatus, 0, 0,
                       &c, 1, 0, trmmsk, 0, 0 );
#else
    if (read(TTY_DESCR, &c, 1) < 0)
      {
          fprintf(stderr,"getkey():  read failed.\n");
          (void) reset_most_tty();
          exit(0);
      }
#endif
	
    return(c); 
}

char getkey()
{
    int i;
    char ch;
    if (!INPUT_BUFFER_LEN) return(sys_getkey());
    ch = INPUT_BUFFER[0];
    INPUT_BUFFER_LEN--;
    for (i = 0; i < INPUT_BUFFER_LEN; i++)
      {
          INPUT_BUFFER[i] = INPUT_BUFFER[i + 1];
      }
    return(ch);
}

void ungetkey(char ch)
{
    int i;
    for (i = 0; i < INPUT_BUFFER_LEN; i++)
      {
          INPUT_BUFFER[i+1] = INPUT_BUFFER[i];
      }
    INPUT_BUFFER[0] = ch;
    INPUT_BUFFER_LEN++;
}
/* character fetch routines */


/* 
 *                          Misc Terminal stuff
 */

/*
**  get the size of the terminal
**  returns 1 on success
*/
int get_term_dimensions(int *cols, int *rows)
{
#ifdef VMS
    int status;
    iosb iostatus;
    $DESCRIPTOR(devnam, "SYS$ERROR");
    ITM_LST itmlst[3] = {
	{sizeof(*rows), DVI$_TT_PAGE, rows, 0},
	{sizeof(*cols), DVI$_DEVBUFSIZ, cols, 0},
	{0,             0,              0,    0}
};
    
    /*
    ** get current terminal characteristics
    */
    status = sys$getdviw(0,           /* Wait on event flag zero  */
                         0,           /* Channel to input terminal  */
                         &devnam,     /* device name */
			 &itmlst,     /* Item descriptor List */
                         &iostatus,   /* Status after operation */
                         0, 0,        /* No AST service   */
                         0);          /* nullarg */
    /*
    ** jump out if bad status
    */
    if ((status&1) == 0)
         return(0);
#else    /* this may need work on other unix-- works for sun4 */
    struct winsize wind_struct;
      
    ioctl(2,TIOCGWINSZ,&wind_struct);
    *cols = (int) wind_struct.ws_col;
    *rows = (int) wind_struct.ws_row;
#endif

    return(1);
}


/*
 *                          Misc UTILITIES
 */

/*
** delete a file from disk
** returns 0 on failure, 1 on success
*/
int sys_delete_file(char *filename)
{
#ifdef VMS
    return (1 + delete(filename));   /* 1 + (0=success or -1=failure) */
#else  /* unix not ready yet */
    return(1 + unlink(filename));
#endif
}

#ifdef VMS
/*
** locate a character in a string
*/
int locate(char ch, char *string)
{
    int i;
    char c;

    i = 0;
    while (c = string[i++], (c != ch) && (c != '\0'));
    if (c == ch) return(i); else return (0);
}

/*
** convert unix type name to VMS name
*/
char *unix2vms(char *file)
{
    int i,device,j,first,last;
    static char vms_name[136];
    char ch;

    if (locate('[',file)) return(file); /* vms_name syntax */
    if (!locate('/',file)) return(file); /* vms_name syntax */

    /* search for the ':' which means a device is present */
    device = locate(':',file);

    i = 0;
    if (device)
      {
          while (ch = file[i], i < device) vms_name[i++] = ch;
      }
    j = i;
    
    /* go from the  end looking for a '/' and mark it */
    i = strlen(file) - 1;
    while(ch = file[i], ch != '/' && i-- >= 0);
    if (ch == '/')
      {
          file[i] = ']';
          last = 0;
      }
    else last = 1;

    i = j;
    vms_name[j++] = '[';
    vms_name[j++] = '.';
    first = 0;
    while(ch = file[i++], ch != '\0')
      {
          switch (ch)
            {
              case '.':
                if (last) vms_name[j++] = '.';
                if (last) break;
                ch = file[i++];
                if (ch == '.')
                  {
                      if (!first) j--;  /* overwrite the dot */
                      vms_name[j++] = '-';
                  }
                else if (ch == '/'); /*  './' combinations-- do nothing */
                else if (ch == ']')
                  {
                      last = 1;
                      if (vms_name[j-1] == '.') j--;
                      vms_name[j++] = ']';
                  }
                
                else vms_name[j++] = '.';
                break;
              case '/':
                if (first)
                  {
                      vms_name[j++] = '.';
                  }
                else
                  {
                      first = 1;
                      /* if '/' is first char or follows a colon do nothing */
                      if ((i!=1) && (file[i-2] != ':'))
                        {
                            vms_name[j++] = '.';
                        }
                      else j--; /* overwrite the '.' following '[' */
                  }
                break;
              case ']':
                last = 1;
                if (vms_name[j-1] == '.') j--;
                vms_name[j++] = ']';
                break;
              default:
                vms_name[j++] = ch;
            }
      }
    return (vms_name);
}

#define NO_STRCASECMP
#endif
#if defined(NO_STRCASECMP) && 0
/*
 * Copyright (c) 1987 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Modified for use on VMS by Earl Fogel, University of Saskatchewan
 * Computing Services, January 1992
 *
 * Added to MOST by Foteos Macrides
 */

typedef unsigned char u_char;

/*
 * This array is designed for mapping upper and lower case letter
 * together for a case independent comparison.  The mappings are
 * based upon ascii character sequences.
 */
static const u_char charmap[] = {
	'\000', '\001', '\002', '\003', '\004', '\005', '\006', '\007',
	'\010', '\011', '\012', '\013', '\014', '\015', '\016', '\017',
	'\020', '\021', '\022', '\023', '\024', '\025', '\026', '\027',
	'\030', '\031', '\032', '\033', '\034', '\035', '\036', '\037',
	'\040', '\041', '\042', '\043', '\044', '\045', '\046', '\047',
	'\050', '\051', '\052', '\053', '\054', '\055', '\056', '\057',
	'\060', '\061', '\062', '\063', '\064', '\065', '\066', '\067',
	'\070', '\071', '\072', '\073', '\074', '\075', '\076', '\077',
	'\100', '\141', '\142', '\143', '\144', '\145', '\146', '\147',
	'\150', '\151', '\152', '\153', '\154', '\155', '\156', '\157',
	'\160', '\161', '\162', '\163', '\164', '\165', '\166', '\167',
	'\170', '\171', '\172', '\133', '\134', '\135', '\136', '\137',
	'\140', '\141', '\142', '\143', '\144', '\145', '\146', '\147',
	'\150', '\151', '\152', '\153', '\154', '\155', '\156', '\157',
	'\160', '\161', '\162', '\163', '\164', '\165', '\166', '\167',
	'\170', '\171', '\172', '\173', '\174', '\175', '\176', '\177',
	'\200', '\201', '\202', '\203', '\204', '\205', '\206', '\207',
	'\210', '\211', '\212', '\213', '\214', '\215', '\216', '\217',
	'\220', '\221', '\222', '\223', '\224', '\225', '\226', '\227',
	'\230', '\231', '\232', '\233', '\234', '\235', '\236', '\237',
	'\240', '\241', '\242', '\243', '\244', '\245', '\246', '\247',
	'\250', '\251', '\252', '\253', '\254', '\255', '\256', '\257',
	'\260', '\261', '\262', '\263', '\264', '\265', '\266', '\267',
	'\270', '\271', '\272', '\273', '\274', '\275', '\276', '\277',
	'\300', '\301', '\302', '\303', '\304', '\305', '\306', '\307',
	'\310', '\311', '\312', '\313', '\314', '\315', '\316', '\317',
	'\320', '\321', '\322', '\323', '\324', '\325', '\326', '\327',
	'\330', '\331', '\332', '\333', '\334', '\335', '\336', '\337',
	'\340', '\341', '\342', '\343', '\344', '\345', '\346', '\347',
	'\350', '\351', '\352', '\353', '\354', '\355', '\356', '\357',
	'\360', '\361', '\362', '\363', '\364', '\365', '\366', '\367',
	'\370', '\371', '\372', '\373', '\374', '\375', '\376', '\377',
};

/*
** case-insensitive strcmp()
*/
int strcasecmp(const char *s1, const char *s2)
{
	register const u_char *cm = charmap,
			*us1 = (const u_char *)s1,
			*us2 = (const u_char *)s2;

	while (cm[*us1] == cm[*us2++])
		if (*us1++ == '\0')
			return (0);
	return (cm[*us1] - cm[*--us2]);
}

/*
** case-insensitive strncmp()
*/
int strncasecmp(const char *s1, const char *s2, register size_t n)
{
	if (n != 0) {
		register const u_char *cm = charmap,
				*us1 = (const u_char *)s1,
				*us2 = (const u_char *)s2;

		do {
			if (cm[*us1] != cm[*us2++])
				return (cm[*us1] - cm[*--us2]);
			if (*us1++ == '\0')
				break;
		} while (--n != 0);
	}
	return (0);
}
#endif /* VMS */


/*
** get day date time year
*/
char *most_get_time(void)
{ 
    time_t clock;
    char *the_time;

    clock = time(NULL);
    the_time = ctime(&clock); /* returns the form Sun Sep 16 01:03:52 1985\n\0 */
    the_time[24] = '\0';
    return(the_time);
}
